@@ -5,7 +5,6 @@ from typing import Any
 import pandas as pd
 
 import ccxt
-
 from ccxt.base.errors import ExchangeError
 from qubx import logger
 from qubx.core.basics import (
@@ -15,7 +14,7 @@ from qubx.core.basics import (
     OrderSide,
 )
 from qubx.core.errors import ErrorLevel, OrderCancellationError, OrderCreationError, create_error_event
-from qubx.core.exceptions import BadRequest, InvalidOrderParameters
+from qubx.core.exceptions import BadRequest, InvalidOrderParameters, OrderNotFound
 from qubx.core.interfaces import (
     IAccountProcessor,
     IBroker,
@@ -43,6 +42,7 @@ class CcxtBroker(IBroker):
         max_cancel_retries: int = 10,
         enable_create_order_ws: bool = False,
         enable_cancel_order_ws: bool = False,
+        enable_edit_order_ws: bool = False,
     ):
         self._exchange_manager = exchange_manager
         self.ccxt_exchange_id = str(self._exchange_manager.exchange.name)
@@ -55,13 +55,13 @@ class CcxtBroker(IBroker):
         self.max_cancel_retries = max_cancel_retries
         self.enable_create_order_ws = enable_create_order_ws
         self.enable_cancel_order_ws = enable_cancel_order_ws
+        self.enable_edit_order_ws = enable_edit_order_ws
 
     @property
     def _loop(self) -> AsyncThreadLoop:
         """Get current AsyncThreadLoop for the exchange."""
         return AsyncThreadLoop(self._exchange_manager.exchange.asyncio_loop)
 
-
     @property
     def is_simulated_trading(self) -> bool:
         return False
@@ -178,7 +178,7 @@ class CcxtBroker(IBroker):
         client_id: str | None = None,
         time_in_force: str = "gtc",
         **options,
-    ) -> Order | None:
+    ) -> Order:
         """
         Submit an order and wait for the result. Exceptions will be raised on errors.
 
@@ -221,23 +221,39 @@ class CcxtBroker(IBroker):
             self._post_order_error_to_databus(
                 err, instrument, order_side, order_type, amount, price, client_id, time_in_force, **options
             )
-            return None
+            raise err
 
-    def cancel_order(self, order_id: str) -> Order | None:
+    def cancel_order(self, order_id: str) -> bool:
+        """Cancel an order synchronously and return success status."""
         orders = self.account.get_orders()
         if order_id not in orders:
             logger.warning(f"Order {order_id} not found in active orders")
-            return None
+            return False
 
         order = orders[order_id]
-        logger.info(f"Canceling order {order_id} ...")
+        logger.info(f"Canceling order {order_id} synchronously...")
 
-        # Submit the cancellation task to the async loop without waiting for the result
+        try:
+            # Submit the task and wait for result
+            future = self._loop.submit(self._cancel_order_with_retry(order_id, order.instrument))
+            return future.result()  # This will block until completion or timeout
+        except Exception as e:
+            logger.error(f"Error during synchronous order cancellation: {e}")
+            return False  # Return False on any error for simplicity
+
+    def cancel_order_async(self, order_id: str) -> None:
+        """Cancel an order asynchronously (non blocking)."""
+        orders = self.account.get_orders()
+        if order_id not in orders:
+            logger.warning(f"Order {order_id} not found in active orders")
+            return
+
+        order = orders[order_id]
+        logger.info(f"Canceling order {order_id} asynchronously...")
+
+        # Submit the task without waiting for result
         self._loop.submit(self._cancel_order_with_retry(order_id, order.instrument))
 
-        # Always return None as requested
-        return None
-
     async def _create_order(
         self,
         instrument: Instrument,
@@ -298,8 +314,22 @@ class CcxtBroker(IBroker):
             logger.warning(f"[<y>{instrument.symbol}</y>] :: Quote is not available for order creation.")
             raise BadRequest(f"Quote is not available for order creation for {instrument.symbol}")
 
-        # TODO: think about automatically setting reduce only when needed
-        if not (reduce_only := options.get("reduceOnly", False)):
+        # Auto-detect if order reduces existing position
+        reduce_only = options.get("reduceOnly", False)
+        if not reduce_only:
+            positions = self.account.get_positions()
+            if instrument in positions:
+                position_qty = positions[instrument].quantity
+                # Check if order closes position AND doesn't exceed position size (which would flip to opposite side)
+                if (position_qty > 0 and order_side == "SELL" and abs(amount) <= abs(position_qty)) or (
+                    position_qty < 0 and order_side == "BUY" and abs(amount) <= abs(position_qty)
+                ):
+                    reduce_only = True
+                    logger.debug(
+                        f"[{instrument.symbol}] Auto-setting reduceOnly=True ({order_side}, position: {position_qty})"
+                    )
+
+        if not reduce_only:
             min_notional = instrument.min_notional
             if min_notional > 0 and abs(amount) * quote.mid_price() < min_notional:
                 raise InvalidOrderParameters(
@@ -364,9 +394,13 @@ class CcxtBroker(IBroker):
         while True:
             try:
                 if self.enable_cancel_order_ws:
-                    await self._exchange_manager.exchange.cancel_order_ws(order_id, symbol=instrument_to_ccxt_symbol(instrument))
+                    await self._exchange_manager.exchange.cancel_order_ws(
+                        order_id, symbol=instrument_to_ccxt_symbol(instrument)
+                    )
                 else:
-                    await self._exchange_manager.exchange.cancel_order(order_id, symbol=instrument_to_ccxt_symbol(instrument))
+                    await self._exchange_manager.exchange.cancel_order(
+                        order_id, symbol=instrument_to_ccxt_symbol(instrument)
+                    )
                 return True
             except ccxt.OperationRejected as err:
                 err_msg = str(err).lower()
@@ -375,8 +409,12 @@ class CcxtBroker(IBroker):
                     # These errors might be temporary if the order is still being processed, so retry
                     logger.debug(f"[{order_id}] Order not found for cancellation, might retry: {err}")
                     # Continue with the retry logic instead of returning immediately
+                # Order cannot be cancelled (e.g., already filled)
+                elif "filled" in err_msg or "partially filled" in err_msg:
+                    logger.debug(f"[{order_id}] Order cannot be cancelled - already executed: {err}")
+                    return False  # FAILURE: Order cannot be cancelled
+                # Other operation rejected errors - don't retry
                 else:
-                    # For other operation rejected errors, don't retry
                     logger.debug(f"[{order_id}] Could not cancel order: {err}")
                     return False
             except (ccxt.NetworkError, ccxt.ExchangeError, ccxt.ExchangeNotAvailable) as e:
@@ -424,8 +462,117 @@ class CcxtBroker(IBroker):
         for order_id in instrument_orders:
             self.cancel_order(order_id)
 
-    def update_order(self, order_id: str, price: float | None = None, amount: float | None = None) -> Order:
-        raise NotImplementedError("Not implemented yet")
+    def update_order(self, order_id: str, price: float, amount: float) -> Order:
+        """Update an existing limit order with new price and amount.
+
+        Args:
+            order_id: The ID of the order to update
+            price: New price for the order (already adjusted by TradingManager)
+            amount: New amount for the order (already adjusted by TradingManager)
+
+        Returns:
+            Order: The updated Order object if successful
+
+        Raises:
+            OrderNotFound: If the order is not found
+            BadRequest: If the order is not a limit order
+            ExchangeError: If the exchange operation fails
+        """
+        active_orders = self.account.get_orders()
+        if order_id not in active_orders:
+            raise OrderNotFound(f"Order {order_id} not found in active orders")
+
+        existing_order = active_orders[order_id]
+
+        # Validate that the order can still be updated (not fully filled/closed)
+        updatable_statuses = ["OPEN", "NEW", "PENDING"]
+        if existing_order.status not in updatable_statuses:
+            raise BadRequest(
+                f"Order {order_id} with status '{existing_order.status}' cannot be updated. "
+                f"Only orders with status {updatable_statuses} can be updated."
+            )
+
+        instrument = existing_order.instrument
+
+        logger.debug(
+            f"[<g>{instrument.symbol}</g>] :: Updating order {order_id}: "
+            f"{amount} @ {price} (was: {existing_order.quantity} @ {existing_order.price} ({existing_order.time_in_force}))"
+        )
+
+        try:
+            # Check if exchange supports order editing
+            if self._exchange_manager.exchange.has.get("editOrder", False):
+                return self._update_order_direct(order_id, existing_order, price, amount)
+            else:
+                return self._update_order_fallback(order_id, existing_order, price, amount)
+        except Exception as err:
+            logger.error(f"Failed to update order {order_id}: {err}")
+            raise
+
+    def _update_order_direct(self, order_id: str, existing_order: Order, price: float, amount: float) -> Order:
+        """Update order using exchange's native edit functionality."""
+        future_result = self._loop.submit(self._edit_order_async(order_id, existing_order, price, amount))
+        updated_order, error = future_result.result()
+
+        if error is not None:
+            raise error
+
+        if updated_order is not None:
+            self.account.process_order(updated_order)
+            logger.debug(f"[<g>{existing_order.instrument.symbol}</g>] :: Successfully updated order {order_id}")
+            return updated_order
+        else:
+            raise Exception("Order update returned None without error")
+
+    def _update_order_fallback(self, order_id: str, existing_order: Order, price: float, amount: float) -> Order:
+        """Update order using cancel+recreate strategy for exchanges without editOrder support."""
+        success = self.cancel_order(order_id)
+        if not success:
+            raise Exception(f"Failed to cancel order {order_id} during update")
+
+        updated_order = self.send_order(
+            instrument=existing_order.instrument,
+            order_side=existing_order.side,
+            order_type=existing_order.type,
+            amount=amount,
+            price=price,
+            client_id=existing_order.client_id,  # Preserve original client_id for tracking
+            time_in_force=existing_order.time_in_force or "gtc",
+        )
+
+        logger.debug(
+            f"[<g>{existing_order.instrument.symbol}</g>] :: Successfully updated order {order_id} -> new order {updated_order.id}"
+        )
+        return updated_order
+
+    async def _edit_order_async(
+        self, order_id: str, existing_order: Order, price: float, amount: float
+    ) -> tuple[Order | None, Exception | None]:
+        """Async helper for direct order editing."""
+        try:
+            ccxt_symbol = instrument_to_ccxt_symbol(existing_order.instrument)
+            ccxt_side = "buy" if existing_order.side == "BUY" else "sell"
+
+            # CCXT requires positive amount (side determines direction)
+            abs_amount = abs(amount)
+
+            # Use WebSocket if enabled, otherwise use REST API
+            if self.enable_edit_order_ws:
+                result = await self._exchange_manager.exchange.edit_order_ws(
+                    id=order_id, symbol=ccxt_symbol, type="limit", side=ccxt_side, amount=abs_amount, price=price, params={}
+                )
+            else:
+                result = await self._exchange_manager.exchange.edit_order(
+                    id=order_id, symbol=ccxt_symbol, type="limit", side=ccxt_side, amount=abs_amount, price=price, params={}
+                )
+
+            # Convert the result back to our Order format
+            updated_order = ccxt_convert_order_info(existing_order.instrument, result)
+            return updated_order, None
+
+        except Exception as err:
+            logger.error(f"Async edit order failed for {order_id}: {err}")
+            return None, err
 
     def exchange(self) -> str:
         """
