@@ -2,13 +2,15 @@ import math
 from typing import Any, Dict, List, Optional
 
 import ccxt.pro as cxp
+from ccxt.async_support.base.ws.client import Client
+from ccxt.base.errors import ExchangeError, InvalidOrder
 from qubx import logger
 
 from ...adapters.polling_adapter import PollingConfig, PollingToWebSocketAdapter
 from ..base import CcxtFuturePatchMixin
 
 # Constants
-FUNDING_RATE_DEFAULT_POLL_MINUTES = 5
+FUNDING_RATE_DEFAULT_POLL_MINUTES = 1
 FUNDING_RATE_HOUR_MS = 60 * 60 * 1000  # 1 hour in milliseconds
 
 
@@ -50,6 +52,67 @@ class HyperliquidEnhanced(CcxtFuturePatchMixin, cxp.hyperliquid):
             0.0,  # bought_volume_quote (not provided by Hyperliquid)
         ]
 
+    def handle_error_message(self, client: Client, message) -> bool:
+        """
+        Override CCXT's handle_error_message to fix the bug where error strings
+        are passed to client.reject() instead of Exception objects.
+
+        This method also adds detailed logging for debugging order failures.
+
+        CCXT Bug: Lines 908, 919, 924 in ccxt/pro/hyperliquid.py pass error strings
+        to client.reject(), but Future.set_exception() requires Exception objects.
+        """
+        # Log the raw message for debugging
+        # logger.debug(f"[Hyperliquid WS Error] Raw message: {self.json(message)}")
+
+        # Check for direct error channel
+        channel = self.safe_string(message, "channel", "")
+        if channel == "error":
+            ret_msg = self.safe_string(message, "data", "")
+            error_msg = f"{self.id} {ret_msg}"
+            logger.error(f"[Hyperliquid WS Error] Channel error: {error_msg}")
+            # FIX: Wrap in Exception instead of passing string
+            client.reject(ExchangeError(error_msg))
+            return True
+
+        # Check response payload for errors
+        data = self.safe_dict(message, "data", {})
+        id = self.safe_string(message, "id")
+        if id is None:
+            id = self.safe_string(data, "id")
+
+        response = self.safe_dict(data, "response", {})
+        payload = self.safe_dict(response, "payload", {})
+
+        # Check for status != 'ok'
+        status = self.safe_string(payload, "status")
+        if status is not None and status != "ok":
+            error_msg = f"{self.id} {self.json(payload)}"
+            logger.error(f"[Hyperliquid WS Error] Status not ok: {error_msg}")
+            # FIX: Wrap in Exception instead of passing string
+            client.reject(InvalidOrder(error_msg), id)
+            return True
+
+        # Check for explicit error type
+        type = self.safe_string(payload, "type")
+        if type == "error":
+            error_msg = f"{self.id} {self.json(payload)}"
+            logger.error(f"[Hyperliquid WS Error] Explicit error type: {error_msg}")
+            # FIX: Wrap in Exception instead of passing string
+            client.reject(ExchangeError(error_msg), id)
+            return True
+
+        # Try standard error handling
+        try:
+            self.handle_errors(0, "", "", "", {}, self.json(payload), payload, {}, {})
+        except Exception as e:
+            logger.error(f"[Hyperliquid WS Error] handle_errors exception: {e}")
+            # This is already an Exception, so it's safe to pass
+            client.reject(e, id)
+            return True
+
+        return False
+
     async def watch_funding_rates(
         self, symbols: Optional[List[str]] = None, params: Optional[Dict[str, Any]] = None
     ) -> Dict[str, Any]:
@@ -200,6 +263,26 @@ class HyperliquidEnhanced(CcxtFuturePatchMixin, cxp.hyperliquid):
                 if cloid and parsed.get("clientOrderId") is None:
                     parsed["clientOrderId"] = cloid
 
+                # Fix timeInForce from tif field
+                # HyperLiquid uses: "Gtc", "Ioc", "Alo"
+                tif = order_info.get("tif")
+                if tif and not parsed.get("timeInForce"):
+                    parsed["timeInForce"] = tif
+                    # logger.debug(f"[HL parse_order] Extracted timeInForce='{tif}' from order {order_info.get('oid')}")
+                # elif not tif:
+                #     oid = order_info.get("oid", "unknown")
+                # logger.warning(
+                #     f"[HL parse_order] No 'tif' field found in order {oid}, raw order_info keys: {list(order_info.keys()) if isinstance(order_info, dict) else 'not a dict'}"
+                # )
+
+                # Fix reduceOnly from reduceOnly field
+                reduce_only = order_info.get("reduceOnly")
+                if reduce_only is not None and not parsed.get("reduceOnly"):
+                    parsed["reduceOnly"] = bool(reduce_only)
+                    logger.debug(
+                        f"[HL parse_order] Extracted reduceOnly={reduce_only} from order {order_info.get('oid')}"
+                    )
+
             # Fix status from HyperLiquid status field
             hl_status = info.get("status")
             if hl_status and parsed.get("status") in [None, "open"]:
