@@ -4,17 +4,20 @@ from datetime import datetime
 from enum import StrEnum
 from queue import Empty, Queue
 from threading import Event, Lock
-from typing import Any, Literal, Optional, TypeAlias, Union
+from typing import TYPE_CHECKING, Any, Literal, Optional, TypeAlias, Union
 
 import numpy as np
 import pandas as pd
 
 from qubx.core.exceptions import QueueTimeout
 from qubx.core.series import Bar, OrderBook, Quote, Trade, time_as_nsec
-from qubx.core.utils import prec_ceil, prec_floor, time_delta_to_str
+from qubx.core.utils import prec_ceil, prec_floor, time_delta_to_str, time_to_str
 from qubx.utils.misc import Stopwatch
 from qubx.utils.ntp import start_ntp_thread, time_now
 
+if TYPE_CHECKING:
+    from qubx.core.interfaces import IStrategyContext
+
 dt_64 = np.datetime64
 td_64 = np.timedelta64
 
@@ -33,6 +36,28 @@ class Liquidation:
     price: float
     side: int
 
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t {self.quantity} @ {self.price} | {self.side}"  # type: ignore
+
+
+@dataclass
+class AggregatedLiquidations:
+    time: dt_64
+    avg_buy_price: float
+    last_buy_price: float
+    buy_amount: float
+    buy_count: int
+    buy_notional: float
+
+    avg_sell_price: float
+    last_sell_price: float
+    sell_amount: float
+    sell_count: int
+    sell_notional: float
+
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t B:{self.buy_amount} @ {self.avg_buy_price} | S:{self.sell_amount} @ {self.avg_sell_price}"  # type: ignore
+
 
 @dataclass
 class FundingRate:
@@ -43,6 +68,9 @@ class FundingRate:
     mark_price: float | None = None
     index_price: float | None = None
 
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t {self.rate:.5f} ({self.interval})"  # type: ignore
+
 
 @dataclass
 class FundingPayment:
@@ -67,6 +95,9 @@ class FundingPayment:
     def funding_rate_apr(self) -> float:
         return self.funding_rate * 365 * 24 / self.funding_interval_hours * 100
 
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t {self.funding_rate:.5f} ({self.funding_interval_hours}H)"  # type: ignore
+
 
 @dataclass
 class OpenInterest:
@@ -81,6 +112,9 @@ class OpenInterest:
     open_interest: float  # Open interest in base asset units
     open_interest_usd: float  # Open interest in USD value
 
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t {self.symbol} | {self.open_interest:.2f} ({self.open_interest_usd:.4f})"  # type: ignore
+
 
 @dataclass
 class TimestampedDict:
@@ -93,6 +127,12 @@ class TimestampedDict:
     time: dt_64
     data: dict[str, Any]
 
+    def __getitem__(self, k: str):
+        return self.data[k]
+
+    def __repr__(self):
+        return f"[{time_to_str(self.time, 'ns')}]\t {str(self.data)}"  # type: ignore
+
 
 class ITimeProvider:
     """
@@ -107,7 +147,17 @@ class ITimeProvider:
 
 
 # Alias for timestamped data types used in Qubx
-Timestamped: TypeAlias = Quote | Trade | Bar | OrderBook | TimestampedDict | FundingRate | Liquidation | FundingPayment
+Timestamped: TypeAlias = (
+    Quote
+    | Trade
+    | Bar
+    | OrderBook
+    | TimestampedDict
+    | FundingRate
+    | Liquidation
+    | FundingPayment
+    | AggregatedLiquidations
+)
 
 
 @dataclass
@@ -248,7 +298,7 @@ class MarketType(StrEnum):
     OPTION = "OPTION"
 
 
-@dataclass
+@dataclass(order=True)
 class Instrument:
     """
     Instrument class.
@@ -279,6 +329,10 @@ class Instrument:
     delist_date: datetime | None = None  # date when instrument is delisted
     inverse: bool = False  # if true, then the future is inverse
 
+    def __post_init__(self):
+        # define how ordering works
+        object.__setattr__(self, "sort_index", f"{self.exchange}:{self.market_type}:{self.symbol}")
+
     @property
     def price_precision(self):
         if not hasattr(self, "_price_precision"):
@@ -403,6 +457,14 @@ class Instrument:
             options=(options or {}) | kwargs,
         )
 
+    def get_amount_for_leverage(self, ctx: "IStrategyContext", leverage: float) -> float:
+        q = ctx.quote(self)
+        capital = ctx.get_total_capital()
+        if q is None or not capital:
+            return 0
+        amount = (capital * leverage) / q.mid_price()
+        return self.round_size_down(amount)
+
     def __hash__(self) -> int:
         return hash((self.symbol, self.exchange, self.market_type))
 
@@ -417,6 +479,28 @@ class Instrument:
     def __repr__(self) -> str:
         return self.__str__()
 
+    def info(self):
+        info_str = f"""
+┌─────────────────────────────┐
+│ Instrument Information      │
+└─────────────────────────────┘
+  Exchange:          {self.exchange}
+  Symbol:            {self.symbol}
+  Market Type:       {self.market_type}
+  Base:              {self.base}
+  Quote:             {self.quote}
+  Exchange Symbol:   {self.exchange_symbol.upper()}
+  Tick Size:         {self.tick_size}
+  Lot Size:          {self.lot_size}
+  Min Size:          {self.min_size}
+  Min Notional:      {self.min_notional}
+  Initial Margin:    {self.initial_margin}
+  Maint. Margin:     {self.maint_margin}
+  Onboard Date:      {self.onboard_date}
+  Delist Date:       {self.delist_date}
+"""
+        print(info_str)
+
 
 class TransactionCostsCalculator:
     """
@@ -568,6 +652,9 @@ class AssetBalance:
         return self
 
 
+DEFAULT_MAINTENANCE_MARGIN = 0.05
+
+
 class Position:
     instrument: Instrument  # instrument for this position
     quantity: float = 0.0  # quantity positive for long and negative for short
@@ -826,7 +913,7 @@ class Position:
         return self.cumulative_funding
 
     def is_open(self) -> bool:
-        return abs(self.quantity) > self.instrument.min_size
+        return abs(self.quantity) >= self.instrument.lot_size
 
     def get_amount_released_funds_after_closing(self, to_remain: float = 0.0) -> float:
         """
@@ -866,10 +953,15 @@ class Position:
         return self.__str__()
 
     def _update_maint_margin(self) -> None:
-        if self.instrument.maint_margin:
-            self.maint_margin = (
-                self.instrument.maint_margin * self._qty_multiplier * abs(self.quantity) * self.last_update_price
-            )
+        # Only apply maintenance margin for leveraged instruments (futures/swaps)
+        # Spot positions don't have margin requirements since you own the actual asset
+        if self.instrument.is_futures():
+            # TODO: could be needed to multiply by qty_multiplier (contract multiplier)
+            # but it needs to be correct and I think for crypto futures it's always 1
+            maint_margin = self.instrument.maint_margin or DEFAULT_MAINTENANCE_MARGIN
+            self.maint_margin = maint_margin * abs(self.quantity) * self.last_update_price
+        else:
+            self.maint_margin = 0.0
 
 
 class CtrlChannel:
@@ -926,12 +1018,14 @@ class DataType(StrEnum):
     OHLC = "ohlc"
     ORDERBOOK = "orderbook"
     LIQUIDATION = "liquidation"
+    AGGREGATED_LIQUIDATIONS = "aggregated_liquidations"
     FUNDING_RATE = "funding_rate"
     FUNDING_PAYMENT = "funding_payment"
     OPEN_INTEREST = "open_interest"
     OHLC_QUOTES = "ohlc_quotes"  # when we want to emulate quotes from OHLC data
     OHLC_TRADES = "ohlc_trades"  # when we want to emulate trades from OHLC data
     RECORD = "record"  # arbitrary timestamped data (actually liquidation and funding rates fall into this type)
+    FUNDAMENTAL = "fundamental"  # fundamental data (with parameters)
 
     def __repr__(self) -> str:
         return self.value
@@ -954,6 +1048,13 @@ class DataType(StrEnum):
                 if not tf:
                     raise ValueError("Timeframe is not provided for OHLC subscription")
                 return f"{self.value}({tf})"
+
+            case DataType.AGGREGATED_LIQUIDATIONS:
+                tf = args[0] if args else kwargs.get("timeframe")
+                if not tf:
+                    raise ValueError("Timeframe is not provided for AGGREGATED_LIQUIDATIONS")
+                return f"{self.value}({tf})"
+
             case DataType.ORDERBOOK:
                 # Check if args is a tuple containing another tuple (the nested case)
                 if len(args) == 1 and isinstance(args[0], tuple):
@@ -971,6 +1072,7 @@ class DataType(StrEnum):
                     tick_size_pct = kwargs.get("tick_size_pct", 0.01)
                     depth = kwargs.get("depth", 200)
                 return f"{self.value}({tick_size_pct}, {depth})"
+
             case DataType.FUNDING_RATE:
                 if len(args) == 0:
                     return f"{self.value}"
@@ -984,6 +1086,18 @@ class DataType(StrEnum):
                         raise ValueError(f"Invalid arguments for FUNDING_RATE subscription: {inner_args}")
                 else:
                     raise ValueError(f"Invalid arguments for FUNDING_RATE subscription: {args}")
+
+            case DataType.FUNDAMENTAL:
+                if len(args) == 0:
+                    return f"{self.value}"
+                else:
+                    inner_args = args[0]
+                    return (
+                        f"{self.value}({', '.join(map(str, *args))})"
+                        if isinstance(inner_args, tuple)
+                        else f"{self.value}({inner_args})"
+                    )
+
             case _:
                 return self.value
 
@@ -1022,6 +1136,11 @@ class DataType(StrEnum):
                     case DataType.OHLC.value:
                         return DataType.OHLC, {"timeframe": time_delta_to_str(pd.Timedelta(params[0]).asm8.item())}
 
+                    case DataType.AGGREGATED_LIQUIDATIONS.value:
+                        return DataType.AGGREGATED_LIQUIDATIONS, {
+                            "timeframe": time_delta_to_str(pd.Timedelta(params[0]).asm8.item())
+                        }
+
                     case DataType.OHLC_QUOTES.value:
                         return DataType.OHLC_QUOTES, {
                             "timeframe": time_delta_to_str(pd.Timedelta(params[0]).asm8.item())
@@ -1043,6 +1162,11 @@ class DataType(StrEnum):
                         else:
                             raise ValueError(f"Invalid arguments for FUNDING_RATE subscription: {params}")
 
+                    case DataType.FUNDAMENTAL.value:
+                        if len(params) > 0:
+                            return DataType.FUNDAMENTAL, {"fields": params}
+                        return DataType.FUNDAMENTAL, {}
+
                     case _:
                         return DataType.NONE, {}
         except IndexError:
