@@ -1,10 +1,17 @@
+import atexit
+import signal
 import traceback
-from threading import Thread
-from typing import Any, Callable
+from functools import wraps
+from threading import Lock, Thread
+from typing import TYPE_CHECKING, Any, Callable
 
 import pandas as pd
 
 from qubx import logger
+
+if TYPE_CHECKING:
+    from qubx.utils.throttler import InstrumentThrottler
+from qubx.core.account import CompositeAccountProcessor
 from qubx.core.basics import (
     AssetBalance,
     CtrlChannel,
@@ -22,6 +29,7 @@ from qubx.core.basics import (
     dt_64,
     td_64,
 )
+from qubx.core.detectors import DelistingDetector
 from qubx.core.errors import BaseErrorEvent, ErrorLevel
 from qubx.core.exceptions import StrategyExceededMaxNumberOfRuntimeFailuresError
 from qubx.core.helpers import (
@@ -41,10 +49,11 @@ from qubx.core.interfaces import (
     IProcessingManager,
     IStrategy,
     IStrategyContext,
-    IStrategyLifecycleNotifier,
+    IStrategyNotifier,
     ISubscriptionManager,
     ITradeDataExport,
     ITradingManager,
+    ITransferManager,
     IUniverseManager,
     PositionsTracker,
     RemovalPolicy,
@@ -64,12 +73,27 @@ from .mixins import (
     UniverseManager,
 )
 
+DEFAULT_POSITION_TRACKER: Callable[[], PositionsTracker] = lambda: PositionsTracker(
+    FixedSizer(1.0, amount_in_quote=False)
+)
+
+
+def check_transfer_manager(func: Callable) -> Callable:
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        if self._transfer_manager is None:
+            raise RuntimeError(
+                "Transfer manager not configured. "
+                "For live mode, set via ctx.initializer.set_transfer_manager() in on_init(). "
+                "For simulation mode, transfer manager is auto-assigned on start()."
+            )
+
+        return func(self, *args, **kwargs)
+
+    return wrapper
+
 
 class StrategyContext(IStrategyContext):
-    DEFAULT_POSITION_TRACKER: Callable[[], PositionsTracker] = lambda: PositionsTracker(
-        FixedSizer(1.0, amount_in_quote=False)
-    )
-
     _market_data_provider: IMarketManager
     _universe_manager: IUniverseManager
     _subscription_manager: ISubscriptionManager
@@ -83,16 +107,26 @@ class StrategyContext(IStrategyContext):
     _scheduler: BasicScheduler
     _initial_instruments: list[Instrument]
     _strategy_name: str
+    _delisting_detector: DelistingDetector
+    _notifier: IStrategyNotifier
+    _aux: DataReader | None
 
     _thread_data_loop: Thread | None = None  # market data loop
     _is_initialized: bool = False
     _exporter: ITradeDataExport | None = None  # Add exporter attribute
-    _lifecycle_notifier: IStrategyLifecycleNotifier | None = None  # Add lifecycle notifier attribute
+    _transfer_manager: ITransferManager | None = None  # Transfer manager for fund transfers
 
     _warmup_positions: dict[Instrument, Position] | None = None
     _warmup_orders: dict[Instrument, list[Order]] | None = None
     _warmup_active_targets: dict[Instrument, list[TargetPosition]] | None = None
 
+    # Shutdown handling
+    _is_stopping: bool = False
+    _stop_lock: Lock
+    _original_sigint_handler: Any = None
+    _original_sigterm_handler: Any = None
+    _atexit_registered: bool = False
+
     def __init__(
         self,
         strategy: IStrategy,
@@ -108,12 +142,13 @@ class StrategyContext(IStrategyContext):
         aux_data_provider: DataReader | None = None,
         exporter: ITradeDataExport | None = None,
         emitter: IMetricEmitter | None = None,
-        lifecycle_notifier: IStrategyLifecycleNotifier | None = None,
+        notifier: IStrategyNotifier | None = None,
         initializer: BasicStrategyInitializer | None = None,
         strategy_name: str | None = None,
         strategy_state: StrategyState | None = None,
         health_monitor: IHealthMonitor | None = None,
         restored_state: RestoredState | None = None,
+        data_throttler: "InstrumentThrottler | None" = None,
     ) -> None:
         self.account = account
         self.strategy = self.__instantiate_strategy(strategy, config)
@@ -137,17 +172,23 @@ class StrategyContext(IStrategyContext):
 
         self._cache = CachedMarketDataHolder()
         self._exporter = exporter
-        self._lifecycle_notifier = lifecycle_notifier
+        self._notifier = notifier if notifier is not None else IStrategyNotifier()
         self._strategy_state = strategy_state if strategy_state is not None else StrategyState()
         self._strategy_name = strategy_name if strategy_name is not None else strategy.__class__.__name__
         self._restored_state = restored_state
+        self._aux = aux_data_provider
 
         self._health_monitor = health_monitor or DummyHealthMonitor()
         self.health = self._health_monitor
 
+        # Initialize shutdown handling
+        self._stop_lock = Lock()
+        self._is_stopping = False
+        self._atexit_registered = False
+
         __position_tracker = self.strategy.tracker(self)
         if __position_tracker is None:
-            __position_tracker = StrategyContext.DEFAULT_POSITION_TRACKER()
+            __position_tracker = DEFAULT_POSITION_TRACKER()
 
         __position_gathering = self.strategy.gatherer(self)
         if __position_gathering is None:
@@ -170,6 +211,13 @@ class StrategyContext(IStrategyContext):
             universe_manager=self,
             aux_data_provider=aux_data_provider,
         )
+
+        # Create delisting detector to be shared between universe and processing managers
+        self._delisting_detector = DelistingDetector(
+            time_provider=self,
+            delisting_check_days=self.initializer.get_delisting_check_days(),
+        )
+
         self._universe_manager = UniverseManager(
             context=self,
             strategy=self.strategy,
@@ -181,6 +229,7 @@ class StrategyContext(IStrategyContext):
             account=self.account,
             position_gathering=__position_gathering,
             warmup_position_gathering=__warmup_position_gathering,
+            delisting_detector=self._delisting_detector,
         )
         self._trading_manager = TradingManager(
             context=self,
@@ -205,6 +254,8 @@ class StrategyContext(IStrategyContext):
             is_simulation=self._data_providers[0].is_simulation,
             exporter=self._exporter,
             health_monitor=self._health_monitor,
+            delisting_detector=self._delisting_detector,
+            data_throttler=data_throttler,
         )
         self.__post_init__()
 
@@ -213,6 +264,8 @@ class StrategyContext(IStrategyContext):
             self.strategy.on_init(self.initializer)
             self._strategy_state.is_on_init_called = True
 
+        self._delisting_detector.delisting_check_days = self.initializer.get_delisting_check_days()
+
         if subscription_warmup := self.initializer.get_subscription_warmup():
             self.set_warmup(subscription_warmup)
 
@@ -244,31 +297,54 @@ class StrategyContext(IStrategyContext):
         stale_data_config = self.initializer.get_stale_data_detection_config()
         self._processing_manager.configure_stale_data_detection(*stale_data_config)
 
+        if self.is_simulation and isinstance(self.account, CompositeAccountProcessor):
+            # Auto-assign simulation transfer manager
+            from qubx.backtester.transfers import SimulationTransferManager
+
+            self._transfer_manager = SimulationTransferManager(self.account, self._time_provider)
+            logger.debug("[StrategyContext] :: Auto-assigned SimulationTransferManager")
+        else:
+            # In live mode, check if strategy set one via initializer
+            self._transfer_manager = self.initializer.get_transfer_manager()
+            if self._transfer_manager is not None:
+                logger.info(f"[StrategyContext] :: Using transfer manager: {type(self._transfer_manager).__name__}")
+
         # - update cache default timeframe
         sub_type = self.get_base_subscription()
         _, params = DataType.from_str(sub_type)
         __default_timeframe = params.get("timeframe", "1sec")
         self._cache.update_default_timeframe(__default_timeframe)
 
+    def _signal_handler(self, signum: int, frame: Any) -> None:
+        """Handle termination signals (SIGINT, SIGTERM) for graceful shutdown."""
+        sig_name = signal.Signals(signum).name
+        logger.info(f"[StrategyContext] :: Received {sig_name} signal - initiating graceful shutdown")
+        self.stop()
+
+    @property
+    def strategy_name(self) -> str:
+        return self._strategy_name or self.strategy.__class__.__name__
+
+    @property
+    def aux(self) -> DataReader | None:
+        return self._aux
+
     def start(self, blocking: bool = False):
         if self._is_initialized:
             raise ValueError("Strategy is already started !")
 
-        # Update initial instruments if strategy set them after warmup
-        if self.get_warmup_positions():
-            self._initial_instruments = list(set(self.get_warmup_positions().keys()) | set(self._initial_instruments))
+        # Register signal handlers for graceful shutdown
+        try:
+            self._original_sigint_handler = signal.signal(signal.SIGINT, self._signal_handler)
+            self._original_sigterm_handler = signal.signal(signal.SIGTERM, self._signal_handler)
+        except (ValueError, OSError) as e:
+            # Signal registration can fail in threads or non-main contexts
+            logger.warning(f"[StrategyContext] :: Could not register signal handlers: {e}")
 
-        # Notify strategy start
-        if self._lifecycle_notifier:
-            try:
-                self._lifecycle_notifier.notify_start(
-                    self._strategy_name,
-                    {
-                        "Instruments": [str(i) for i in self._initial_instruments],
-                    },
-                )
-            except Exception as e:
-                logger.error(f"[StrategyContext] :: Failed to notify strategy start: {e}")
+        # Register atexit handler as backup for abnormal termination
+        if not self._atexit_registered:
+            atexit.register(self.stop)
+            self._atexit_registered = True
 
         # - run cron scheduler
         self._scheduler.run()
@@ -283,6 +359,29 @@ class StrategyContext(IStrategyContext):
         # - start health metrics monitor
         self._health_monitor.start()
 
+        # Update initial instruments if strategy set them after warmup
+        if self.get_warmup_positions():
+            self._initial_instruments = list(set(self.get_warmup_positions().keys()) | set(self._initial_instruments))
+
+        # Add open positions to initial instruments
+        open_positions = {k: p for k, p in self.get_positions().items() if p.is_open()}
+        self._initial_instruments = list(set(open_positions.keys()) | set(self._initial_instruments))
+
+        # Notify strategy start
+        if self._notifier and not self.is_simulation:
+            try:
+                self._notifier.notify_start(
+                    {
+                        "Exchanges": "|".join(self.exchanges),
+                        "Total Capital": f"${self.get_total_capital():,.0f}",
+                        "Open Positions": len(open_positions),
+                        "Instruments": len(self._initial_instruments),
+                        "Mode": "Paper" if self.is_paper_trading else "Live",
+                    },
+                )
+            except Exception as e:
+                logger.error(f"[StrategyContext] :: Failed to notify strategy start: {e}")
+
         # - update universe with initial instruments after the strategy is initialized
         self.set_universe(self._initial_instruments, skip_callback=True)
 
@@ -297,24 +396,46 @@ class StrategyContext(IStrategyContext):
         self._is_initialized = True
 
     def stop(self):
+        """
+        Stop the strategy context with robust shutdown handling.
+
+        Features:
+        - Double-stop prevention with lock
+        - Priority-based cleanup (critical paths first)
+        - Fault-tolerant (exceptions don't block other cleanup)
+        - Thread cleanup with timeout
+        - Signal handler and atexit cleanup
+        """
+        # Prevent concurrent or repeated stops
+        with self._stop_lock:
+            if self._is_stopping:
+                logger.debug("[StrategyContext] :: Stop already in progress, skipping duplicate call")
+                return
+            self._is_stopping = True
+
+        # PRIORITY 1: Critical path - always execute notifier and on_stop
+        # These are the most important callbacks that must always run
+
         # Notify strategy stop
-        if self._lifecycle_notifier:
+        if self._notifier and not self.is_simulation:
             try:
-                self._lifecycle_notifier.notify_stop(
-                    self._strategy_name,
+                self._notifier.notify_stop(
                     {
-                        "Total Capital": f"{self.get_total_capital():.2f}",
+                        "Total Capital": f"{self.get_total_capital():,.0f}",
                         "Net Leverage": f"{self.get_net_leverage():.2%}",
                         "Positions": len([p for i, p in self.get_positions().items() if abs(p.quantity) > i.min_size]),
+                        "Mode": "Paper" if self.is_paper_trading else "Live",
                     },
                 )
             except Exception as e:
                 logger.error(f"[StrategyContext] :: Failed to notify strategy stop: {e}")
+                logger.opt(colors=False).error(traceback.format_exc())
 
-        # - invoke strategy's stop code
+        # Invoke strategy's stop code
         try:
             if not self.is_warmup_in_progress:
                 self.strategy.on_stop(self)
+                logger.debug("[StrategyContext] :: Strategy on_stop() completed")
         except Exception as strat_error:
             logger.error(
                 f"[<y>StrategyContext</y>] :: Strategy {self._strategy_name} raised an exception in on_stop: {strat_error}"
@@ -322,29 +443,88 @@ class StrategyContext(IStrategyContext):
             logger.opt(colors=False).error(traceback.format_exc())
 
             # Notify strategy error
-            if self._lifecycle_notifier:
+            if self._notifier:
                 try:
-                    self._lifecycle_notifier.notify_error(self._strategy_name, strat_error)
+                    self._notifier.notify_error(strat_error)
                 except Exception as e:
                     logger.error(f"[StrategyContext] :: Failed to notify strategy error: {e}")
 
+        # PRIORITY 2: Stop data providers and thread
+
+        # Close data providers
         if self._thread_data_loop:
-            for data_provider in self._data_providers:
-                data_provider.close()
+            try:
+                for data_provider in self._data_providers:
+                    try:
+                        data_provider.close()
+                        logger.debug(f"[StrategyContext] :: Closed data provider: {type(data_provider).__name__}")
+                    except Exception as e:
+                        logger.error(
+                            f"[StrategyContext] :: Failed to close data provider {type(data_provider).__name__}: {e}"
+                        )
+            except Exception as e:
+                logger.error(f"[StrategyContext] :: Error iterating data providers: {e}")
 
-            # - we assume that the channel is the same for all data providers
-            self._data_providers[0].channel.stop()
-            self._thread_data_loop.join()
-            self._thread_data_loop = None
+            # Stop the channel
+            try:
+                self._data_providers[0].channel.stop()
+            except Exception as e:
+                logger.error(f"[StrategyContext] :: Failed to stop data channel: {e}")
 
-        # - stop account processing
-        self.account.stop()
+            # Join thread with timeout
+            try:
+                thread_timeout = 30.0  # 30 seconds timeout
+                self._thread_data_loop.join(timeout=thread_timeout)
+                if self._thread_data_loop.is_alive():
+                    logger.warning(
+                        f"[StrategyContext] :: Data loop thread did not stop within {thread_timeout}s timeout - may still be running"
+                    )
+                else:
+                    logger.debug("[StrategyContext] :: Data loop thread stopped gracefully")
+                self._thread_data_loop = None
+            except Exception as e:
+                logger.error(f"[StrategyContext] :: Error joining data loop thread: {e}")
 
-        # - stop health metrics monitor
-        self._health_monitor.stop()
+        # PRIORITY 3: Stop account processing
+        try:
+            self.account.stop()
+        except Exception as e:
+            logger.error(f"[StrategyContext] :: Failed to stop account processor: {e}")
+            logger.opt(colors=False).error(traceback.format_exc())
 
-        # - close logging
-        self._logging.close()
+        # PRIORITY 4: Stop health metrics monitor
+        try:
+            self._health_monitor.stop()
+        except Exception as e:
+            logger.error(f"[StrategyContext] :: Failed to stop health monitor: {e}")
+            logger.opt(colors=False).error(traceback.format_exc())
+
+        # PRIORITY 5: Close logging
+        try:
+            self._logging.close()
+        except Exception as e:
+            logger.error(f"[StrategyContext] :: Failed to close logging: {e}")
+            logger.opt(colors=False).error(traceback.format_exc())
+
+        # CLEANUP: Restore signal handlers and deregister atexit
+        try:
+            if self._original_sigint_handler is not None:
+                signal.signal(signal.SIGINT, self._original_sigint_handler)
+                self._original_sigint_handler = None
+            if self._original_sigterm_handler is not None:
+                signal.signal(signal.SIGTERM, self._original_sigterm_handler)
+                self._original_sigterm_handler = None
+        except Exception as e:
+            logger.warning(f"[StrategyContext] :: Failed to restore signal handlers: {e}")
+
+        try:
+            if self._atexit_registered:
+                atexit.unregister(self.stop)
+                self._atexit_registered = False
+        except Exception as e:
+            logger.warning(f"[StrategyContext] :: Failed to unregister atexit handler: {e}")
+
+        logger.info("[StrategyContext] :: Strategy context stopped")
 
     def is_running(self):
         return self._thread_data_loop is not None and self._thread_data_loop.is_alive()
@@ -357,6 +537,10 @@ class StrategyContext(IStrategyContext):
     def is_paper_trading(self) -> bool:
         return self._brokers[0].is_simulated_trading
 
+    @property
+    def notifier(self) -> IStrategyNotifier:
+        return self._notifier
+
     # IAccountViewer delegation
 
     # capital information
@@ -383,8 +567,8 @@ class StrategyContext(IStrategyContext):
             positions.update(self.account.get_positions(e))
         return positions
 
-    def get_orders(self, instrument: Instrument | None = None) -> dict[str, Order]:
-        return self.account.get_orders(instrument)
+    def get_orders(self, instrument: Instrument | None = None, exchange: str | None = None) -> dict[str, Order]:
+        return self.account.get_orders(instrument, exchange)
 
     def position_report(self, exchange: str | None = None) -> dict:
         return self.account.position_report(exchange)
@@ -462,18 +646,36 @@ class StrategyContext(IStrategyContext):
     ) -> Order:
         return self._trading_manager.set_target_position(instrument, target, price, **options)
 
+    def set_target_leverage(
+        self, instrument: Instrument, leverage: float, price: float | None = None, **options
+    ) -> None:
+        return self._trading_manager.set_target_leverage(instrument, leverage, price, **options)
+
     def close_position(self, instrument: Instrument, without_signals: bool = False) -> None:
         return self._trading_manager.close_position(instrument, without_signals)
 
     def close_positions(self, market_type: MarketType | None = None, without_signals: bool = False) -> None:
         return self._trading_manager.close_positions(market_type, without_signals)
 
-    def cancel_order(self, order_id: str, exchange: str | None = None) -> None:
+    def cancel_order(self, order_id: str, exchange: str | None = None) -> bool:
+        """Cancel a specific order synchronously."""
         return self._trading_manager.cancel_order(order_id, exchange)
 
-    def cancel_orders(self, instrument: Instrument):
+    def cancel_order_async(self, order_id: str, exchange: str | None = None) -> None:
+        """Cancel a specific order asynchronously (non blocking)."""
+        return self._trading_manager.cancel_order_async(order_id, exchange)
+
+    def cancel_orders(self, instrument: Instrument) -> None:
+        """Cancel all orders for an instrument."""
         return self._trading_manager.cancel_orders(instrument)
 
+    def update_order(self, order_id: str, price: float, amount: float, exchange: str | None = None) -> Order:
+        """Update an existing limit order with new price and amount."""
+        return self._trading_manager.update_order(order_id, price, amount, exchange)
+
+    def get_min_size(self, instrument: Instrument, amount: float | None = None) -> float:
+        return self._trading_manager.get_min_size(instrument, amount)
+
     # IUniverseManager delegation
     def set_universe(
         self, instruments: list[Instrument], skip_callback: bool = False, if_has_position_then: RemovalPolicy = "close"
@@ -563,16 +765,12 @@ class StrategyContext(IStrategyContext):
     def emit_signal(self, signal: Signal | list[Signal]) -> None:
         return self._processing_manager.emit_signal(signal)
 
-    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> None:
-        """
-        Register a custom method to be called at specified times.
-
-        Args:
-            cron_schedule: Cron-like schedule string (e.g., "0 0 * * *" for daily at midnight)
-            method: Method to call when schedule triggers
-        """
+    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> str:
         return self._processing_manager.schedule(cron_schedule, method)
 
+    def unschedule(self, event_id: str) -> bool:
+        return self._processing_manager.unschedule(event_id)
+
     # IWarmupStateSaver delegation
     def set_warmup_positions(self, positions: dict[Instrument, Position]) -> None:
         self._warmup_positions = positions
@@ -595,6 +793,22 @@ class StrategyContext(IStrategyContext):
     def get_restored_state(self) -> RestoredState | None:
         return self._restored_state
 
+    # ITransferManager delegation methods
+    @check_transfer_manager
+    def transfer_funds(self, from_exchange: str, to_exchange: str, currency: str, amount: float) -> str:
+        assert self._transfer_manager is not None
+        return self._transfer_manager.transfer_funds(from_exchange, to_exchange, currency, amount)
+
+    @check_transfer_manager
+    def get_transfer_status(self, transaction_id: str) -> dict[str, Any]:
+        assert self._transfer_manager is not None
+        return self._transfer_manager.get_transfer_status(transaction_id)
+
+    @check_transfer_manager
+    def get_transfers(self) -> dict[str, dict[str, Any]]:
+        assert self._transfer_manager is not None
+        return self._transfer_manager.get_transfers()
+
     # private methods
     def __process_incoming_data_loop(self, channel: CtrlChannel):
         logger.info("[StrategyContext] :: Start processing market data")
@@ -608,21 +822,15 @@ class StrategyContext(IStrategyContext):
                     self._health_monitor.record_start_processing(d_type, dt_64(data.time, "ns"))
 
                 # - notify error if error level is medium or higher
-                if (
-                    self._lifecycle_notifier
-                    and isinstance(data, BaseErrorEvent)
-                    and data.level.value >= ErrorLevel.MEDIUM.value
-                ):
-                    self._lifecycle_notifier.notify_error(
-                        self._strategy_name, data.error or Exception("Unknown error"), {"message": str(data)}
-                    )
+                if self._notifier and isinstance(data, BaseErrorEvent) and data.level.value >= ErrorLevel.MEDIUM.value:
+                    self._notifier.notify_error(data.error or Exception("Unknown error"), {"message": str(data)})
 
                 if self.process_data(instrument, d_type, data, hist):
                     channel.stop()
                     break
 
                 if _should_record:
-                    self._health_monitor.record_end_processing(d_type, dt_64(data.time, "ns"))
+                    self._health_monitor.record_end_processing(d_type, dt_64(data.time, "ns"))  # type: ignore
 
             except StrategyExceededMaxNumberOfRuntimeFailuresError:
                 channel.stop()
@@ -630,8 +838,8 @@ class StrategyContext(IStrategyContext):
             except Exception as e:
                 logger.error(f"Error processing market data: {e}")
                 logger.opt(colors=False).error(traceback.format_exc())
-                if self._lifecycle_notifier:
-                    self._lifecycle_notifier.notify_error(self._strategy_name, e)
+                if self._notifier:
+                    self._notifier.notify_error(e)
                 # Don't stop the channel here, let it continue processing
 
         logger.info("[StrategyContext] :: Market data processing stopped")
