@@ -12,9 +12,10 @@ This module includes:
     - Strategy lifecycle notifiers
 """
 
+import inspect
 import traceback
 from dataclasses import dataclass
-from typing import Any, Callable, Literal, Protocol, runtime_checkable
+from typing import TYPE_CHECKING, Any, Callable, Literal, Protocol, runtime_checkable
 
 import numpy as np
 import pandas as pd
@@ -45,6 +46,9 @@ from qubx.core.errors import BaseErrorEvent
 from qubx.core.helpers import set_parameters_to_object
 from qubx.core.series import OHLCV, Bar, Quote
 
+if TYPE_CHECKING:
+    from qubx.data.readers import DataReader
+
 RemovalPolicy = Literal["close", "wait_for_close", "wait_for_change"]
 
 
@@ -174,11 +178,12 @@ class IAccountViewer:
         """
         return self.get_positions()
 
-    def get_orders(self, instrument: Instrument | None = None) -> dict[str, Order]:
-        """Get active orders, optionally filtered by instrument.
+    def get_orders(self, instrument: Instrument | None = None, exchange: str | None = None) -> dict[str, Order]:
+        """Get active orders, optionally filtered by instrument and/or exchange.
 
         Args:
             instrument: Optional instrument to filter orders by
+            exchange: Optional exchange to filter orders by
 
         Returns:
             dict[str, Order]: Dictionary mapping order IDs to Order objects
@@ -277,6 +282,170 @@ class IAccountViewer:
         return 0.0
 
 
+class IExchangeExtensions:
+    """
+    Base class for exchange-specific API extensions.
+
+    Provides automatic method discovery using Python's reflection capabilities.
+    All public methods (not starting with '_') are automatically discoverable
+    with their signatures and docstrings.
+    """
+
+    def list_methods(self) -> dict[str, str]:
+        """
+        List all available extension methods with brief descriptions.
+
+        Returns:
+            dict[str, str]: Dictionary mapping method names to their first-line descriptions
+        """
+        methods = {}
+        for name, method in inspect.getmembers(self, predicate=inspect.ismethod):
+            # Skip private methods and the reflection methods themselves
+            if name.startswith("_") or name in ["list_methods", "call_method", "get_method_signature", "help"]:
+                continue
+
+            # Get first line of docstring as description
+            doc = inspect.getdoc(method)
+            description = doc.split("\n")[0] if doc else "No description"
+            methods[name] = description
+
+        return methods
+
+    def call_method(self, method_name: str, *args, **kwargs) -> Any:
+        """
+        Dynamically call an extension method by name.
+
+        Args:
+            method_name: Name of the method to call
+            *args: Positional arguments to pass to the method
+            **kwargs: Keyword arguments to pass to the method
+
+        Returns:
+            Any: The result of the method call
+
+        Raises:
+            AttributeError: If the method doesn't exist
+            TypeError: If invalid arguments are provided
+        """
+        if not hasattr(self, method_name):
+            available = ", ".join(self.list_methods().keys())
+            raise AttributeError(
+                f"Extension method '{method_name}' not found. Available methods: {available if available else 'none'}"
+            )
+
+        method = getattr(self, method_name)
+        if not callable(method) or method_name.startswith("_"):
+            raise AttributeError(f"'{method_name}' is not a callable extension method")
+
+        return method(*args, **kwargs)
+
+    def get_method_signature(self, method_name: str) -> dict[str, Any]:
+        """
+        Get detailed information about a method's signature and documentation.
+
+        Args:
+            method_name: Name of the method to inspect
+
+        Returns:
+            dict: Dictionary containing:
+                - 'signature': String representation of the method signature
+                - 'description': Full docstring of the method
+                - 'parameters': List of parameter names and their annotations
+                - 'return_type': Return type annotation (if available)
+
+        Raises:
+            AttributeError: If the method doesn't exist
+        """
+        if not hasattr(self, method_name):
+            raise AttributeError(f"Extension method '{method_name}' not found")
+
+        method = getattr(self, method_name)
+        sig = inspect.signature(method)
+        doc = inspect.getdoc(method)
+
+        # Extract parameter information
+        params = []
+        for param_name, param in sig.parameters.items():
+            if param_name == "self":
+                continue
+
+            param_info = {
+                "name": param_name,
+                "annotation": str(param.annotation) if param.annotation != param.empty else None,
+                "default": str(param.default) if param.default != param.empty else None,
+            }
+
+            params.append(param_info)
+
+        return {
+            "signature": str(sig),
+            "description": doc if doc else "No documentation available",
+            "parameters": params,
+            "return_type": str(sig.return_annotation) if sig.return_annotation != sig.empty else None,
+        }
+
+    def help(self, method_name: str | None = None) -> str:
+        """
+        Get formatted help text for extension methods.
+
+        Args:
+            method_name: Optional name of specific method to get help for.
+                        If None, lists all available methods.
+
+        Returns:
+            str: Formatted help text
+        """
+        if method_name is None:
+            # List all methods
+            methods = self.list_methods()
+            if not methods:
+                return "No extension methods available"
+
+            lines = ["Available extension methods:"]
+            for name, desc in sorted(methods.items()):
+                lines.append(f"  {name}: {desc}")
+            return "\n".join(lines)
+        else:
+            # Detailed help for specific method
+            try:
+                info = self.get_method_signature(method_name)
+                lines = [f"Method: {method_name}{info['signature']}", "", info["description"]]
+                return "\n".join(lines)
+            except AttributeError as e:
+                return str(e)
+
+
+class EmptyExchangeExtensions(IExchangeExtensions):
+    """
+    Default empty implementation of exchange extensions.
+
+    Used for exchanges that don't have specific extension methods.
+    Ensures all brokers always have an extensions property without needing hasattr checks.
+    """
+
+    def __init__(self, exchange_name: str):
+        """
+        Initialize empty extensions.
+
+        Args:
+            exchange_name: Name of the exchange (for error messages)
+        """
+        self.exchange_name = exchange_name
+
+    def list_methods(self) -> dict[str, str]:
+        """Return empty dictionary - no methods available."""
+        return {}
+
+    def call_method(self, method_name: str, *args, **kwargs) -> Any:
+        """
+        Raise NotImplementedError for any method call.
+
+        Raises:
+            NotImplementedError: Always, with message about the exchange not having extensions
+        """
+        raise NotImplementedError(f"Exchange '{self.exchange_name}' does not have extension methods")
+
+
 class IBroker:
     """Broker provider interface for managing trading operations.
 
@@ -285,6 +454,19 @@ class IBroker:
 
     channel: CtrlChannel
 
+    @property
+    def extensions(self) -> IExchangeExtensions:
+        """
+        Access exchange-specific API extensions.
+
+        Returns EmptyExchangeExtensions by default. Specific broker implementations
+        can override this property to provide their own extensions.
+
+        Returns:
+            IExchangeExtensions: Exchange-specific extensions (always present, never None)
+        """
+        return EmptyExchangeExtensions(self.exchange())
+
     @property
     def is_simulated_trading(self) -> bool:
         """
@@ -348,13 +530,24 @@ class IBroker:
         """
         raise NotImplementedError("send_order_async is not implemented")
 
-    def cancel_order(self, order_id: str) -> None:
-        """Cancel an existing order (non blocking).
+    def cancel_order(self, order_id: str) -> bool:
+        """Cancel an existing order synchronously.
+
+        Args:
+            order_id: The ID of the order to cancel.
+
+        Returns:
+            bool: True if cancellation was successful, False otherwise.
+        """
+        raise NotImplementedError("cancel_order is not implemented")
+
+    def cancel_order_async(self, order_id: str) -> None:
+        """Cancel an existing order asynchronously (non blocking).
 
         Args:
             order_id: The ID of the order to cancel.
         """
-        raise NotImplementedError("cancel_order is not implemented")
+        raise NotImplementedError("cancel_order_async is not implemented")
 
     def cancel_orders(self, instrument: Instrument) -> None:
         """Cancel all orders for an instrument.
@@ -364,8 +557,8 @@ class IBroker:
         """
         raise NotImplementedError("cancel_orders is not implemented")
 
-    def update_order(self, order_id: str, price: float | None = None, amount: float | None = None) -> Order:
-        """Update an existing order.
+    def update_order(self, order_id: str, price: float, amount: float) -> Order:
+        """Update an existing order with new price and amount.
 
         Args:
             order_id: The ID of the order to update.
@@ -378,7 +571,8 @@ class IBroker:
         Raises:
             NotImplementedError: If the method is not implemented
             OrderNotFound: If the order is not found
-            BadRequest: If the request is invalid
+            BadRequest: If the request is invalid (e.g., not a limit order)
+            InvalidOrderParameters: If the order cannot be updated
         """
         raise NotImplementedError("update_order is not implemented")
 
@@ -479,7 +673,7 @@ class IDataProvider:
         """
         ...
 
-    def get_quote(self, instrument: Instrument) -> Quote:
+    def get_quote(self, instrument: Instrument) -> Quote | None:
         """
         Get the latest quote for an instrument.
         """
@@ -698,11 +892,24 @@ class ITradingManager:
         """Close all positions."""
         ...
 
-    def cancel_order(self, order_id: str, exchange: str | None = None) -> None:
-        """Cancel a specific order.
+    def cancel_order(self, order_id: str, exchange: str | None = None) -> bool:
+        """Cancel a specific order synchronously.
 
         Args:
             order_id: ID of the order to cancel
+            exchange: Exchange to cancel on (optional)
+
+        Returns:
+            bool: True if cancellation was successful, False otherwise.
+        """
+        ...
+
+    def cancel_order_async(self, order_id: str, exchange: str | None = None) -> None:
+        """Cancel a specific order asynchronously (non blocking).
+
+        Args:
+            order_id: ID of the order to cancel
+            exchange: Exchange to cancel on (optional)
         """
         ...
 
@@ -714,6 +921,34 @@ class ITradingManager:
         """
         ...
 
+    def update_order(self, order_id: str, price: float, amount: float, exchange: str | None = None) -> Order:
+        """Update an existing limit order with new price and amount.
+
+        Args:
+            order_id: ID of the order to update
+            price: New price for the order
+            amount: New amount for the order
+            exchange: Exchange to update on (optional, defaults to first exchange)
+
+        Returns:
+            Order: The updated order object
+
+        Raises:
+            OrderNotFound: If the order is not found
+            BadRequest: If the order is not a limit order or other validation errors
+            InvalidOrderParameters: If the update parameters are invalid
+        """
+        ...
+
+    def get_min_size(self, instrument: Instrument, amount: float | None = None) -> float:
+        """Get the minimum size for an instrument.
+
+        Args:
+            instrument: The instrument to get the minimum size for
+            amount: The amount to be traded to determine if it's position reducing or not
+        """
+        ...
+
     def exchanges(self) -> list[str]: ...
 
 
@@ -1023,6 +1258,43 @@ class IAccountProcessor(IAccountViewer):
         ...
 
 
+class ITransferManager:
+    """Manages transfer operations between exchanges."""
+
+    def transfer_funds(self, from_exchange: str, to_exchange: str, currency: str, amount: float) -> str:
+        """
+        Transfer funds between exchanges.
+
+        Args:
+            from_exchange: Source exchange identifier
+            to_exchange: Destination exchange identifier
+            currency: Currency to transfer
+            amount: Amount to transfer
+
+        Returns:
+            str: Transaction ID
+        """
+        ...
+
+    def get_transfer_status(self, transaction_id: str) -> dict[str, Any]:
+        """
+        Get the status of a transfer.
+
+        Args:
+            transaction_id: Transaction ID
+
+        Returns:
+            dict[str, Any]: Transfer status
+        """
+        ...
+
+    def get_transfers(self) -> dict[str, dict[str, Any]]:
+        """
+        Get all transfers.
+        """
+        ...
+
+
 class IProcessingManager:
     """Manages event processing."""
 
@@ -1049,6 +1321,7 @@ class IProcessingManager:
     def set_event_schedule(self, schedule: str) -> None:
         """
         Set the schedule for triggering events (default is to only trigger on data events).
+        Replaces any existing event schedule.
         """
         ...
 
@@ -1084,7 +1357,7 @@ class IProcessingManager:
         """
         ...
 
-    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> None:
+    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> str:
         """
         Register a custom method to be called at specified times.
 
@@ -1094,6 +1367,18 @@ class IProcessingManager:
         """
         ...
 
+    def unschedule(self, event_id: str) -> bool:
+        """
+        Unschedule a scheduled event.
+
+        Args:
+            event_id: ID of the event to unschedule
+
+        Returns:
+            bool: True if event was found and unscheduled, False otherwise
+        """
+        ...
+
     def configure_stale_data_detection(
         self, enabled: bool, detection_period: str | None = None, check_interval: str | None = None
     ) -> None:
@@ -1162,12 +1447,16 @@ class IStrategyContext(
     IProcessingManager,
     IAccountViewer,
     IWarmupStateSaver,
+    ITransferManager,
 ):
     strategy: "IStrategy"
     initializer: "IStrategyInitializer"
     account: IAccountProcessor
     emitter: "IMetricEmitter"
     health: "IHealthReader"
+    notifier: "IStrategyNotifier"
+    strategy_name: str
+    aux: "DataReader | None"
 
     _strategy_state: StrategyState
 
@@ -1946,6 +2235,50 @@ class IStrategyInitializer:
         """
         ...
 
+    def set_delisting_check_days(self, days: int) -> None:
+        """
+        Set the number of days ahead to check for delisting.
+
+        When an instrument has a delist_date set, this configuration determines how many
+        days before the delisting date the framework should:
+        1. Filter the instrument out from universe updates
+        2. Close positions and remove the instrument during the daily delisting check
+
+        Args:
+            days: Number of days ahead to check for delisting (default: 1)
+        """
+        ...
+
+    def get_delisting_check_days(self) -> int:
+        """
+        Get the number of days ahead to check for delisting.
+
+        Returns:
+            int: Number of days ahead to check for delisting
+        """
+        ...
+
+    def set_transfer_manager(self, manager: ITransferManager) -> None:
+        """
+        Set the transfer manager for handling fund transfers between exchanges.
+
+        This is typically used in live mode to inject a transfer service client.
+        In simulation mode, a transfer manager is automatically assigned.
+
+        Args:
+            manager: Transfer manager implementation
+        """
+        ...
+
+    def get_transfer_manager(self) -> ITransferManager | None:
+        """
+        Get the configured transfer manager.
+
+        Returns:
+            ITransferManager | None: The transfer manager if set, None otherwise
+        """
+        ...
+
 
 class IStrategy(metaclass=Mixable):
     """Base class for trading strategies."""
@@ -2023,6 +2356,16 @@ class IStrategy(metaclass=Mixable):
         """
         return None
 
+    def on_deals(self, ctx: IStrategyContext, deals: list[Deal]) -> None:
+        """
+        Called when deals are received.
+
+        Args:
+            ctx: Strategy context.
+            deals: The deals.
+        """
+        return None
+
     def on_error(self, ctx: IStrategyContext, error: BaseErrorEvent) -> None:
         """
         Called when an error occurs.
@@ -2042,6 +2385,29 @@ class IStrategy(metaclass=Mixable):
     def gatherer(self, ctx: IStrategyContext) -> IPositionGathering | None:
         pass
 
+    def get_dashboard_data(self, ctx: IStrategyContext) -> dict[str, Any] | None:
+        """
+        Provide custom data for the Textual UI dashboard.
+
+        This method is called periodically when running in Textual mode to populate
+        custom dashboard panels. Return a dictionary with any JSON-serializable data
+        you want to display.
+
+        Args:
+            ctx: Strategy context
+
+        Returns:
+            Dictionary with custom dashboard data, or None if no custom data
+
+        Example:
+            return {
+                "signal_strength": self.current_signal,
+                "volatility_regime": "high" if self.vol > threshold else "low",
+                "open_alerts": len(self.alerts),
+            }
+        """
+        return None
+
 
 class IMetricEmitter:
     """Interface for emitting metrics to external monitoring systems."""
@@ -2050,7 +2416,7 @@ class IMetricEmitter:
         self,
         name: str,
         value: float,
-        tags: dict[str, str] | None = None,
+        tags: dict[str, Any] | None = None,
         timestamp: dt_64 | None = None,
         instrument: Instrument | None = None,
     ) -> None:
@@ -2092,15 +2458,16 @@ class IMetricEmitter:
         """
         pass
 
-    def set_time_provider(self, time_provider: ITimeProvider) -> None:
+    def set_context(self, context: "IStrategyContext") -> None:
         """
-        Set the time provider for the metric emitter.
+        Set the strategy context for the metric emitter.
 
-        This method is used to set the time provider that will be used to get timestamps
-        when no explicit timestamp is provided in the emit method.
+        This method is used to set the context that provides access to time and simulation state.
+        The context is used to automatically add is_live tag and get timestamps when no explicit
+        timestamp is provided in the emit method.
 
         Args:
-            time_provider: The time provider to use
+            context: The strategy context to use
         """
         pass
 
@@ -2124,31 +2491,50 @@ class IMetricEmitter:
         """
         pass
 
+    def emit_deals(
+        self,
+        time: dt_64,
+        instrument: Instrument,
+        deals: list[Deal],
+        account: "IAccountViewer",
+    ) -> None:
+        """
+        Emit deals to the monitoring system.
 
-class IStrategyLifecycleNotifier:
+        This method is called to emit executed deals for monitoring and analysis purposes.
+        It has to be manually called by the strategy, context does not call it automatically.
+
+        Args:
+            time: Timestamp when the deals were generated
+            instrument: Instrument the deals belong to
+            deals: List of deals to emit
+            account: Account viewer to get account information like total capital, leverage, etc.
+        """
+        pass
+
+
+class IStrategyNotifier:
     """Interface for notifying about strategy lifecycle events."""
 
-    def notify_start(self, strategy_name: str, metadata: dict[str, Any] | None = None) -> None:
+    def notify_start(self, metadata: dict[str, Any] | None = None) -> None:
         """
         Notify that a strategy has started.
 
         Args:
-            strategy_name: Name of the strategy that started
             metadata: Optional dictionary with additional information about the start event
         """
         pass
 
-    def notify_stop(self, strategy_name: str, metadata: dict[str, Any] | None = None) -> None:
+    def notify_stop(self, metadata: dict[str, Any] | None = None) -> None:
         """
         Notify that a strategy has stopped.
 
         Args:
-            strategy_name: Name of the strategy that stopped
             metadata: Optional dictionary with additional information about the stop event
         """
         pass
 
-    def notify_error(self, strategy_name: str, error: Exception, metadata: dict[str, Any] | None = None) -> None:
+    def notify_error(self, error: Exception, metadata: dict[str, Any] | None = None) -> None:
         """
         Notify that a strategy has encountered an error.
 
@@ -2158,3 +2544,14 @@ class IStrategyLifecycleNotifier:
             metadata: Optional dictionary with additional information about the error
         """
         pass
+
+    def notify_message(self, message: str, metadata: dict[str, Any] | None = None, **kwargs: Any) -> None:
+        """
+        Notify that a strategy has encountered an error.
+
+        Args:
+            message: The message to notify
+            metadata: Optional dictionary with additional information about the message
+            **kwargs: Additional keyword arguments
+        """
+        pass
