@@ -5,7 +5,7 @@ from copy import copy
 from io import BytesIO
 from itertools import chain
 from pathlib import Path
-from typing import Any, Callable, Literal
+from typing import Any, Callable, Literal, cast
 
 import matplotlib
 import matplotlib.pylab as plt
@@ -175,7 +175,7 @@ def cagr(returns, periods=DAILY):
 
     cumrets = (returns + 1).cumprod(axis=0)
     years = len(cumrets) / float(periods)
-    return (cumrets.iloc[-1] ** (1.0 / years)) - 1.0
+    return ((cumrets.iloc[-1] ** (1.0 / years)) - 1.0) * 100
 
 
 def calmar_ratio(returns, periods=DAILY):
@@ -606,6 +606,7 @@ class TradingSessionResult:
     executions_log: pd.DataFrame                     # executed trades
     signals_log: pd.DataFrame                        # signals generated by the strategy
     targets_log: pd.DataFrame                        # targets generated by the strategy
+    transfers_log: pd.DataFrame | None = None        # fund transfers between exchanges (if any)
     strategy_class: str                              # strategy full qualified class name
     parameters: dict[str, Any]                       # strategy parameters if provided
     is_simulation: bool
@@ -640,6 +641,7 @@ class TradingSessionResult:
         author: str | None = None,
         variation_name: str | None = None,
         emitter_data: pd.DataFrame | None = None,
+        transfers_log: pd.DataFrame | None = None,
     ):
         self.id = id
         self.name = name
@@ -654,6 +656,7 @@ class TradingSessionResult:
         self.executions_log = executions_log
         self.signals_log = signals_log
         self.targets_log = targets_log
+        self.transfers_log = transfers_log
         self.strategy_class = strategy_class
         self.parameters = parameters if parameters else {}
         self.is_simulation = is_simulation
@@ -679,6 +682,82 @@ class TradingSessionResult:
         pft_total["Total_Commissions"] = pft_total["Total_Commissions"].cumsum()
         return self.get_total_capital() + pft_total["Total_PnL"] - pft_total["Total_Commissions"] * commission_factor
 
+    def get_equity_per_exchange(self, commission_factor: float = 1, with_transfers: bool = True) -> pd.DataFrame:
+        """Get equity curve per exchange
+
+        Args:
+            commission_factor: Factor to apply to commissions (default 1.0)
+            with_transfers: Whether to account for transfers between exchanges (default True)
+
+        Returns:
+            DataFrame with exchange names as columns and equity curves as values
+        """
+        if self.portfolio_log.empty:
+            return pd.DataFrame()
+
+        # Single exchange case - just use get_equity
+        if len(self.exchanges) == 1:
+            return pd.DataFrame({self.exchanges[0]: self.get_equity(commission_factor)})
+
+        result = {}
+
+        # Process each exchange
+        for exchange in self.exchanges:
+            # Filter columns for this exchange (pattern: "EXCHANGE:SYMBOL_Metric")
+            exchange_prefix = f"{exchange}:"
+            exchange_cols = [col for col in self.portfolio_log.columns if col.startswith(exchange_prefix)]
+
+            if not exchange_cols:
+                # No activity on this exchange - return zeros
+                result[exchange] = pd.Series(0.0, index=self.portfolio_log.index)
+                continue
+
+            # Extract portfolio log for this exchange and calculate total PnL
+            exchange_log = cast(pd.DataFrame, self.portfolio_log[exchange_cols])
+            pft_total = calculate_total_pnl(exchange_log, split_cumulative=False)
+            pft_total["Total_PnL"] = pft_total["Total_PnL"].cumsum()
+            pft_total["Total_Commissions"] = pft_total["Total_Commissions"].cumsum()
+
+            # Get initial capital for this exchange
+            if isinstance(self.capital, dict):
+                init_capital = self.capital.get(exchange, 0.0)
+            else:
+                init_capital = self.capital
+
+            # Calculate base equity
+            equity = init_capital + pft_total["Total_PnL"] - pft_total["Total_Commissions"] * commission_factor
+
+            # Account for transfers if requested
+            if with_transfers and self.transfers_log is not None and not self.transfers_log.empty:
+                if (
+                    "to_exchange" in self.transfers_log.columns
+                    and "from_exchange" in self.transfers_log.columns
+                    and "amount" in self.transfers_log.columns
+                ):
+                    # Filter transfers for this exchange
+                    inbound = self.transfers_log[self.transfers_log["to_exchange"] == exchange]["amount"]
+                    outbound = self.transfers_log[self.transfers_log["from_exchange"] == exchange]["amount"]
+
+                    # Create series of transfer amounts at each timestamp
+                    transfer_amounts = pd.Series(0.0, index=self.portfolio_log.index)
+
+                    # Add inbound transfers
+                    for ts, amount in inbound.items():
+                        if ts in transfer_amounts.index:
+                            transfer_amounts.loc[ts] += amount
+
+                    # Subtract outbound transfers
+                    for ts, amount in outbound.items():
+                        if ts in transfer_amounts.index:
+                            transfer_amounts.loc[ts] -= amount
+
+                    # Cumulative sum to apply transfers from their timestamp onwards
+                    equity += transfer_amounts.cumsum()
+
+            result[exchange] = equity
+
+        return pd.DataFrame(result)
+
     @property
     def drawdown_pct(self) -> pd.Series:
         """Get drawdown as percentage over time"""
@@ -747,6 +826,11 @@ class TradingSessionResult:
         """Get number of executions"""
         return len(self.executions_log)
 
+    @property
+    def turnover(self) -> float:
+        """Get average daily turnover as percentage of equity"""
+        return self.performance().get("avg_daily_turnover", 0.0)
+
     @property
     def leverage(self) -> pd.Series:
         """Get leverage over time"""
@@ -779,7 +863,7 @@ class TradingSessionResult:
             for k in [
                 "equity", "drawdown_usd", "drawdown_pct",
                 "compound_returns", "returns_daily", "returns", "monthly_returns",
-                "rolling_sharpe", "long_value", "short_value",
+                "rolling_sharpe", "long_value", "short_value", "turnover",
             ]:
                 self._metrics.pop(k, None)
             # fmt: on
@@ -896,7 +980,8 @@ class TradingSessionResult:
             perf = {
                 "cagr": 0.0,
                 "sharpe": 0.0,
-                "max_dd_pct": 0.0,
+                "mdd_pct": 0.0,
+                "daily_turnover": 0.0,
                 "gain": 0.0,
                 "calmar": 0.0,
                 "sortino": 0.0,
@@ -916,7 +1001,8 @@ class TradingSessionResult:
             "performance": {
                 "cagr": perf.get("cagr", 0.0),
                 "sharpe": perf.get("sharpe", 0.0),
-                "max_dd_pct": perf.get("max_dd_pct", 0.0),
+                "mdd_pct": perf.get("mdd_pct", 0.0),
+                "daily_turnover": perf.get("daily_turnover", 0.0),
                 "total_return": perf.get("gain", 0.0) / self.get_total_capital() * 100
                 if self.get_total_capital() > 0
                 else 0.0,
@@ -1001,6 +1087,8 @@ class TradingSessionResult:
         self.executions_log.to_csv(p / "executions.csv")
         self.signals_log.to_csv(p / "signals.csv")
         self.targets_log.to_csv(p / "targets.csv")
+        if self.transfers_log is not None and not self.transfers_log.empty:
+            self.transfers_log.to_csv(p / "transfers.csv")
 
         # - save report
         with open(p / "report.html", "w") as f:
@@ -1053,7 +1141,7 @@ class TradingSessionResult:
         params = info["parameters"]
 
         # - performance extracting
-        _dd_mtrx = ["max_dd_pct", "mdd_usd", "mdd_start", "mdd_peak", "mdd_recover"]
+        _dd_mtrx = ["mdd_pct", "mdd_usd", "mdd_start", "mdd_peak", "mdd_recover"]
         perf_main = {"".join(list(map(str.capitalize, c.split("_")))): v for c, v in perf.items() if c not in _dd_mtrx}
         perf_dd = {"".join(list(map(str.capitalize, c.split("_")))): v for c, v in perf.items() if c in _dd_mtrx}
         perf_dd["MddStart"] = pd.Timestamp(perf_dd["MddStart"]).strftime("%Y-%m-%d %H:%M:%S")
@@ -1129,22 +1217,36 @@ description: {_desc}
             info = yaml.safe_load(zip_ref.read("info.yml"))
             try:
                 portfolio = pd.read_csv(
-                    zip_ref.open("portfolio.csv"), index_col=["timestamp"], parse_dates=["timestamp"]
+                    zip_ref.open("portfolio.csv"),
+                    index_col=["timestamp"],
+                    parse_dates=["timestamp"],
+                    date_format="mixed",
                 )
             except:
                 portfolio = pd.DataFrame()
             try:
                 executions = pd.read_csv(
-                    zip_ref.open("executions.csv"), index_col=["timestamp"], parse_dates=["timestamp"]
+                    zip_ref.open("executions.csv"),
+                    index_col=["timestamp"],
+                    parse_dates=["timestamp"],
+                    date_format="mixed",
                 )
             except:
                 executions = pd.DataFrame()
             try:
-                signals = pd.read_csv(zip_ref.open("signals.csv"), index_col=["timestamp"], parse_dates=["timestamp"])
+                signals = pd.read_csv(
+                    zip_ref.open("signals.csv"),
+                    index_col=["timestamp"],
+                    parse_dates=["timestamp"],
+                    date_format="mixed",
+                    low_memory=False,
+                )
             except:
                 signals = pd.DataFrame()
             try:
-                targets = pd.read_csv(zip_ref.open("targets.csv"), index_col=["timestamp"], parse_dates=["timestamp"])
+                targets = pd.read_csv(
+                    zip_ref.open("targets.csv"), index_col=["timestamp"], parse_dates=["timestamp"], date_format="mixed"
+                )
             except:
                 targets = pd.DataFrame()
 
@@ -1381,16 +1483,21 @@ def portfolio_metrics(
     execs = len(executions_log)
     mdd_pct = 100 * dd_data / equity.cummax() if execs > 0 else pd.Series(0, index=equity.index)
     sheet["equity"] = equity
-    sheet["gain"] = sheet["equity"].iloc[-1] - sheet["equity"].iloc[0]
-    sheet["cagr"] = cagr(returns_daily, performance_statistics_period)
     sheet["sharpe"] = sharpe_ratio(returns_daily, risk_free, performance_statistics_period)
+    sheet["cagr"] = cagr(returns_daily, performance_statistics_period)
+
+    # turnover calculation
+    symbols = list(set(portfolio_log.columns.str.split("_").str.get(0).values))
+    turnover_series = calculate_turnover(portfolio_log, symbols, equity, resample="1d")
+    sheet["turnover"] = turnover_series
+    sheet["daily_turnover"] = turnover_series.mean() if len(turnover_series) > 0 else 0.0
+
     sheet["qr"] = qr(equity) if execs > 0 else 0
-    sheet["drawdown_usd"] = dd_data
+    sheet["mdd_pct"] = max(mdd_pct)
     sheet["drawdown_pct"] = mdd_pct
+    sheet["drawdown_usd"] = dd_data
     # 25-May-2019: MDE fixed Max DD pct calculations
-    sheet["max_dd_pct"] = max(mdd_pct)
-    # sheet["max_dd_pct_on_init"] = 100 * mdd / init_cash
-    sheet["mdd_usd"] = mdd
+    # sheet["mdd_pct_on_init"] = 100 * mdd / init_cash
     sheet["mdd_start"] = equity.index[ddstart]
     sheet["mdd_peak"] = equity.index[ddpeak]
     sheet["mdd_recover"] = equity.index[ddrecover]
@@ -1403,12 +1510,12 @@ def portfolio_metrics(
     )
     sheet["calmar"] = calmar_ratio(returns_daily, performance_statistics_period)
     # sheet["ann_vol"] = annual_volatility(returns_daily)
-    sheet["tail_ratio"] = tail_ratio(returns_daily)
-    sheet["stability"] = stability_of_returns(returns_daily)
+    # sheet["tail_ratio"] = tail_ratio(returns_daily)
+    # sheet["stability"] = stability_of_returns(returns_daily)
     sheet["monthly_returns"] = aggregate_returns(returns_daily, convert_to="mon")
     r_m = np.mean(returns_daily)
     r_s = np.std(returns_daily)
-    sheet["var"] = var_cov_var(init_cash, r_m, r_s)
+    # sheet["var"] = var_cov_var(init_cash, r_m, r_s)
     sheet["avg_return"] = 100 * r_m
 
     # portfolio market values
@@ -1416,6 +1523,8 @@ def portfolio_metrics(
     sheet["long_value"] = mkt_value[mkt_value > 0].sum(axis=1).fillna(0)
     sheet["short_value"] = mkt_value[mkt_value < 0].sum(axis=1).fillna(0)
 
+    sheet["gain"] = sheet["equity"].iloc[-1] - sheet["equity"].iloc[0]
+    sheet["mdd_usd"] = mdd
     # total commissions
     sheet["fees"] = pft_total["Total_Commissions"].iloc[-1]
 
@@ -1423,9 +1532,10 @@ def portfolio_metrics(
     funding_columns = pft_total.filter(regex=".*_Funding")
     if not funding_columns.empty:
         total_funding = funding_columns.sum(axis=1)
-        sheet["funding_pnl"] = 100 * total_funding.iloc[-1] / init_cash  # as percentage of initial capital
-    else:
-        sheet["funding_pnl"] = 0.0
+        if total_funding.iloc[-1] != 0:
+            sheet["funding_pnl"] = 100 * total_funding.iloc[-1] / init_cash  # as percentage of initial capital
+    # else:
+    #     sheet["funding_pnl"] = 0.0
 
     # executions metrics
     sheet["execs"] = execs
@@ -1568,7 +1678,7 @@ def tearsheet(
             report = report.set_index("id", append=True).swaplevel()
             if sort_by:
                 report = report.sort_values(by=sort_by, ascending=sort_ascending)
-            return report
+            return report.round(2)
 
     else:
         return _tearsheet_single(
@@ -1725,7 +1835,7 @@ def _tearsheet_single(
             ay = sbp(_n, 5)
             plt.plot(lev, c="c", lw=1.5, label="Leverage")
         plt.subplots_adjust(hspace=0)
-        return pd.DataFrame(report).T.round(3)
+        return pd.DataFrame(report).T.round(2)
 
 
 def calculate_leverage(
@@ -1828,6 +1938,53 @@ def calculate_pnl_per_symbol(
     return df
 
 
+def calculate_turnover(
+    portfolio_log: pd.DataFrame,
+    symbols: list[str],
+    equity: pd.Series,
+    resample: str = "1d",
+) -> pd.Series:
+    """
+    Calculate daily turnover as percentage of equity.
+
+    Turnover measures trading activity by calculating the absolute value of position changes
+    multiplied by price, then dividing by equity.
+
+    Args:
+        portfolio_log: Portfolio log dataframe with position and price columns
+        symbols: List of symbols to calculate turnover for
+        equity: Equity curve series
+        resample: Resampling period for turnover calculation (default "1d")
+
+    Returns:
+        pd.Series: Daily turnover as percentage of equity
+    """
+    position_diffs = []
+
+    for symbol in symbols:
+        pos_col = f"{symbol}_Pos"
+        price_col = f"{symbol}_Price"
+
+        if pos_col in portfolio_log.columns and price_col in portfolio_log.columns:
+            # Calculate absolute position change multiplied by price (notional value)
+            position_diff = portfolio_log[pos_col].diff().abs() * portfolio_log[price_col]
+            position_diffs.append(position_diff)
+
+    if not position_diffs:
+        return pd.Series(0, index=equity.index)
+
+    # Sum all position changes and resample to specified period
+    notional_turnover = pd.concat(position_diffs, axis=1).sum(axis=1).resample(resample).sum()
+
+    # Resample equity to match turnover frequency
+    equity_resampled = equity.resample(resample).last()
+
+    # Calculate turnover as percentage of equity
+    daily_turnover = notional_turnover.div(equity_resampled).mul(100).fillna(0)
+
+    return daily_turnover
+
+
 def chart_signals(
     result: TradingSessionResult,
     symbol: str,
