@@ -3,11 +3,12 @@ import uuid
 from collections import defaultdict
 from multiprocessing.pool import ThreadPool
 from types import FunctionType
-from typing import Any, Callable
-
-import pandas as pd
+from typing import TYPE_CHECKING, Any, Callable
 
 from qubx import logger
+
+if TYPE_CHECKING:
+    from qubx.utils.throttler import InstrumentThrottler
 from qubx.core.basics import (
     DataType,
     Deal,
@@ -24,6 +25,7 @@ from qubx.core.basics import (
     dt_64,
     td_64,
 )
+from qubx.core.detectors import DelistingDetector, StaleDataDetector
 from qubx.core.errors import BaseErrorEvent
 from qubx.core.exceptions import StrategyExceededMaxNumberOfRuntimeFailuresError
 from qubx.core.helpers import BasicScheduler, CachedMarketDataHolder, process_schedule_spec
@@ -43,7 +45,6 @@ from qubx.core.interfaces import (
 )
 from qubx.core.loggers import StrategyLogging
 from qubx.core.series import Bar, OrderBook, Quote, Trade
-from qubx.core.stale_data_detector import StaleDataDetector
 from qubx.trackers.riskctrl import _InitializationStageTracker
 
 
@@ -67,6 +68,8 @@ class ProcessingManager(IProcessingManager):
     _exporter: ITradeDataExport | None = None
     _health_monitor: IHealthMonitor
     _stale_data_detector: StaleDataDetector
+    _delisting_detector: DelistingDetector
+    _data_throttler: "InstrumentThrottler | None"
 
     _handlers: dict[str, Callable[["ProcessingManager", Instrument, str, Any], TriggerEvent | None]]
     _strategy_name: str
@@ -109,7 +112,9 @@ class ProcessingManager(IProcessingManager):
         scheduler: BasicScheduler,
         is_simulation: bool,
         health_monitor: IHealthMonitor,
+        delisting_detector: DelistingDetector,
         exporter: ITradeDataExport | None = None,
+        data_throttler: "InstrumentThrottler | None" = None,
     ):
         self._context = context
         self._strategy = strategy
@@ -126,6 +131,8 @@ class ProcessingManager(IProcessingManager):
         self._scheduler = scheduler
         self._exporter = exporter
         self._health_monitor = health_monitor
+        self._delisting_detector = delisting_detector
+        self._data_throttler = data_throttler
 
         # Initialize stale data detector with default disabled state
         # Will be configured later based on strategy settings
@@ -161,26 +168,32 @@ class ProcessingManager(IProcessingManager):
         self._scheduler.schedule_event("30 23 * * *", "delisting_check")
 
     def set_fit_schedule(self, schedule: str) -> None:
+        logger.info(f"Setting fit schedule to {schedule}")
         rule = process_schedule_spec(schedule)
         if rule.get("type") != "cron":
             raise ValueError("Only cron type is supported for fit schedule")
+        self._scheduler.unschedule_event("fit")
         self._scheduler.schedule_event(rule["schedule"], "fit")
 
     def set_event_schedule(self, schedule: str) -> None:
+        logger.info(f"Setting event schedule to {schedule}")
         rule = process_schedule_spec(schedule)
         if not rule or "type" not in rule:
-            raise ValueError(f"Can't recognoize schedule format: '{schedule}'")
+            raise ValueError(f"Can't recognize schedule format: '{schedule}'")
 
         if rule["type"] != "cron":
             raise ValueError("Only cron type is supported for event schedule")
 
+        # Unschedule existing time event first to avoid conflicts
+        self._scheduler.unschedule_event("time")
+
         self._scheduler.schedule_event(rule["schedule"], "time")
         self._trigger_on_time_event = True
 
     def get_event_schedule(self, event_id: str) -> str | None:
         return self._scheduler.get_schedule_for_event(event_id)
 
-    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> None:
+    def schedule(self, cron_schedule: str, method: Callable[["IStrategyContext"], None]) -> str:
         """
         Register a custom method to be called at specified times.
 
@@ -201,6 +214,11 @@ class ProcessingManager(IProcessingManager):
         # Schedule the event
         self._scheduler.schedule_event(rule["schedule"], event_id)
 
+        return event_id
+
+    def unschedule(self, event_id: str) -> bool:
+        return self._scheduler.unschedule_event(event_id)
+
     def configure_stale_data_detection(
         self, enabled: bool, detection_period: str | None = None, check_interval: str | None = None
     ) -> None:
@@ -238,6 +256,17 @@ class ProcessingManager(IProcessingManager):
         return self._context._strategy_state.is_on_fit_called
 
     def __process_data(self, instrument: Instrument, d_type: str, data: Any, is_historical: bool) -> bool:
+        # Apply throttling for live data (not historical)
+        # Don't throttle order/deals/system events - only market data
+        if (
+            not is_historical
+            and self._data_throttler is not None
+            and d_type not in ["order", "deals", "time", "fit", "delisting_check", "service_time"]
+            and not self._data_throttler.should_send(d_type, instrument)
+        ):
+            # Data throttled - skip processing
+            return False
+
         handler = self._handlers.get(d_type)
         with self._health_monitor("process_data"):
             if not d_type:
@@ -701,7 +730,7 @@ class ProcessingManager(IProcessingManager):
             try:
                 method = self._custom_scheduled_methods[event_type]
                 method(self._context)
-                logger.debug(f"[ProcessingManager] :: Executed custom scheduled method for event: {event_type}")
+                # logger.debug(f"[ProcessingManager] :: Executed custom scheduled method for event: {event_type}")
             except Exception as e:
                 logger.error(
                     f"[ProcessingManager] :: Error executing custom scheduled method for event {event_type}: {e}"
@@ -824,29 +853,23 @@ class ProcessingManager(IProcessingManager):
         self, instrument: Instrument | None, event_type: str, data: tuple[dt_64 | None, dt_64]
     ) -> None:
         """
-        Daily delisting check - close positions for instruments delisting within 1 day.
+        Daily delisting check - close positions for instruments delisting within configured days.
         This is a system-wide scheduled event, so instrument will be None.
         """
         if not self._is_data_ready():
             return
 
-        logger.debug("Performing daily delisting check")
+        logger.debug(
+            f"Performing daily delisting check (checking {self._delisting_detector.delisting_check_days} days ahead)"
+        )
 
-        current_time = data[1]
-        current_timestamp = pd.Timestamp(current_time, unit="ns")
-        one_day_ahead = current_timestamp + pd.Timedelta(days=1)
-
-        # Find instruments delisting within 1 day
-        instruments_to_close = []
-        for instr in self._context.instruments:
-            if instr.delist_date is not None:
-                delist_timestamp = pd.Timestamp(instr.delist_date).tz_localize(None)
-                if delist_timestamp <= one_day_ahead:
-                    instruments_to_close.append(instr)
+        # Find instruments delisting within configured days
+        instruments_to_close = self._delisting_detector.detect_delistings(self._context.instruments)
 
         if instruments_to_close:
             logger.info(
-                f"Found {len(instruments_to_close)} instruments scheduled for delisting: {instruments_to_close}"
+                f"Found {len(instruments_to_close)} instruments scheduled for delisting: "
+                f"{[instr.symbol for instr in instruments_to_close]}"
             )
 
             # Force close positions and remove from universe
@@ -925,6 +948,19 @@ class ProcessingManager(IProcessingManager):
             # - notify universe manager about position change
             self._universe_manager.on_alter_position(instrument)
 
+            # - emit deals to metric emitters if available
+            if self._context.emitter is not None and deals:
+                self._context.emitter.emit_deals(
+                    time=self._time_provider.time(),
+                    instrument=instrument,
+                    deals=deals,
+                    account=self._account,
+                )
+
+            # - notify strategy about deals
+            if deals:
+                self._strategy.on_deals(self._context, deals)
+
             # - process active targets: if we got 0 position after executions remove current position from active
             if not self._context.get_position(instrument).is_open():
                 self._active_targets.pop(instrument, None)
