@@ -8,5 +8,12 @@ before it is exported to external systems.
 from qubx.exporters.formatters.base import DefaultFormatter, IExportFormatter
 from qubx.exporters.formatters.incremental import IncrementalFormatter
 from qubx.exporters.formatters.slack import SlackMessageFormatter
+from qubx.exporters.formatters.target_position import TargetPositionFormatter
 
-__all__ = ["IExportFormatter", "DefaultFormatter", "SlackMessageFormatter", "IncrementalFormatter"]
+__all__ = [
+    "IExportFormatter",
+    "DefaultFormatter",
+    "SlackMessageFormatter",
+    "IncrementalFormatter",
+    "TargetPositionFormatter",
+]
