@@ -447,7 +447,12 @@ class AsyncThreadLoop:
         self.loop = loop
 
     def submit(self, coro: Awaitable) -> concurrent.futures.Future:
-        return asyncio.run_coroutine_threadsafe(coro, self.loop)
+        return asyncio.run_coroutine_threadsafe(coro, self.loop)  # type: ignore
+
+    async def run_in_executor(
+        self, executor: concurrent.futures.Executor, func: Callable, *args, **kwargs
+    ) -> concurrent.futures.Future:
+        return await self.loop.run_in_executor(executor, func, *args, **kwargs)
 
 
 def synchronized(func: Callable):
@@ -582,3 +587,18 @@ def load_qubx_resources_as_text(path: Path | str) -> str:
         raise e
 
     return data
+
+
+def install_uvloop():
+    """Install uvloop as the default event loop implementation."""
+    import uvloop
+
+    from qubx import logger
+
+    try:
+        uvloop.install()
+        logger.debug("uvloop installed successfully")
+    except ImportError:
+        logger.warning("uvloop not available, using default asyncio event loop")
+    except Exception as e:
+        logger.warning(f"Failed to install uvloop: {e}, using default asyncio event loop")
