@@ -1,8 +1,10 @@
+import asyncio
 import socket
 import time
 from collections import defaultdict
 from functools import reduce
 from pathlib import Path
+from typing import cast
 
 import pandas as pd
 
@@ -23,6 +25,12 @@ from qubx.backtester.utils import (
 from qubx.connectors.ccxt.data import CcxtDataProvider
 from qubx.connectors.ccxt.factory import get_ccxt_account, get_ccxt_broker, get_ccxt_exchange_manager
 from qubx.connectors.tardis.data import TardisDataProvider
+from qubx.connectors.xlighter.factory import (
+    get_xlighter_account,
+    get_xlighter_broker,
+    get_xlighter_client,
+    get_xlighter_data_provider,
+)
 from qubx.core.account import CompositeAccountProcessor
 from qubx.core.basics import (
     CtrlChannel,
@@ -46,18 +54,21 @@ from qubx.core.interfaces import (
 from qubx.core.loggers import StrategyLogging
 from qubx.core.lookups import lookup
 from qubx.core.mixins.utils import EXCHANGE_MAPPINGS
+from qubx.data.helpers import CachedPrefetchReader
 from qubx.health import BaseHealthMonitor
 from qubx.loggers import create_logs_writer
 from qubx.restarts.state_resolvers import StateResolver
 from qubx.restarts.time_finders import TimeFinder
 from qubx.restorers import create_state_restorer
-from qubx.utils.misc import class_import, green, makedirs, red
+from qubx.utils.misc import class_import, green, install_uvloop, makedirs, red
 from qubx.utils.runner.configs import (
     ExchangeConfig,
     LoggingConfig,
+    PrefetchConfig,
     ReaderConfig,
     RestorerConfig,
     StrategyConfig,
+    TypedReaderConfig,
     WarmupConfig,
     load_strategy_config_from_yaml,
     resolve_aux_config,
@@ -66,8 +77,8 @@ from qubx.utils.runner.factory import (
     construct_aux_reader,
     create_data_type_readers,
     create_exporters,
-    create_lifecycle_notifiers,
     create_metric_emitters,
+    create_notifiers,
 )
 
 from .accounts import AccountConfigurationManager
@@ -75,6 +86,24 @@ from .accounts import AccountConfigurationManager
 INVERSE_EXCHANGE_MAPPINGS = {mapping: exchange for exchange, mapping in EXCHANGE_MAPPINGS.items()}
 
 
+def _cleanup_event_loop(loop: asyncio.AbstractEventLoop | None) -> None:
+    """
+    Cleanup the shared event loop.
+
+    Args:
+        loop: The event loop to cleanup. If None, does nothing.
+    """
+    if loop is None:
+        return
+
+    try:
+        if not loop.is_closed():
+            loop.call_soon_threadsafe(loop.stop)
+            logger.debug("Shared event loop stopped")
+    except Exception as e:
+        logger.warning(f"Failed to cleanup event loop: {e}")
+
+
 def run_strategy_yaml(
     config_file: Path,
     account_file: Path | None = None,
@@ -181,6 +210,15 @@ def run_strategy(
     if not config.live:
         raise ValueError("Live configuration is required for strategy execution")
 
+    # Install uvloop and create shared event loop
+    install_uvloop()
+    # loop = asyncio.new_event_loop()
+    # loop_thread = Thread(target=loop.run_forever, daemon=True, name="SharedEventLoop")
+    # loop_thread.start()
+    # logger.debug("Shared event loop started in background thread")
+    # Separate loop is created for each exchange connector to avoid conflicts
+    loop = None
+
     QubxLogConfig.setup_logger(
         level=QubxLogConfig.get_log_level(),
         custom_formatter=(simulated_formatter := SimulatedLogFormatter(LiveTimeProvider())).formatter,
@@ -202,6 +240,7 @@ def run_strategy(
         simulated_formatter=simulated_formatter,
         no_color=no_color,
         aux_configs=aux_configs,
+        loop=loop,
     )
 
     try:
@@ -210,15 +249,17 @@ def run_strategy(
             restored_state=restored_state,
             exchanges=config.live.exchanges,
             warmup=config.live.warmup,
-            aux_configs=aux_configs,
+            prefetch_config=config.live.prefetch,
             simulated_formatter=simulated_formatter,
-            enable_funding=config.simulation.enable_funding if config.simulation else False,
+            enable_funding=config.live.warmup.enable_funding if config.live.warmup else False,
         )
     except KeyboardInterrupt:
         logger.info("Warmup interrupted by user")
+        _cleanup_event_loop(loop)
         return ctx
     except Exception as e:
         logger.error(f"Warmup failed: {e}")
+        _cleanup_event_loop(loop)
         raise e
 
     # Start the strategy context
@@ -229,6 +270,7 @@ def run_strategy(
             logger.info("Stopped by user")
         finally:
             ctx.stop()
+            _cleanup_event_loop(loop)
     else:
         ctx.start()
 
@@ -263,6 +305,7 @@ def create_strategy_context(
     simulated_formatter: SimulatedLogFormatter,
     no_color: bool = False,
     aux_configs: list[ReaderConfig] | None = None,
+    loop: asyncio.AbstractEventLoop | None = None,
 ) -> IStrategyContext:
     """
     Create a strategy context from the given configuration.
@@ -286,16 +329,11 @@ def create_strategy_context(
 
     _logging = _setup_strategy_logging(stg_name, config.live.logging, simulated_formatter, run_id)
 
-    # Use provided aux_configs or resolve if not provided (for backwards compatibility)
-    if aux_configs is None:
-        aux_configs = resolve_aux_config(config.aux, getattr(config.live, "aux", None))
-    _aux_reader = construct_aux_reader(aux_configs)
-
     # Create metric emitters with run_id as a tag
     _metric_emitter = create_metric_emitters(config.live.emission, stg_name, run_id) if config.live.emission else None
 
     # Create lifecycle notifiers
-    _lifecycle_notifier = create_lifecycle_notifiers(config.live.notifiers, stg_name) if config.live.notifiers else None
+    _notifier = create_notifiers(config.live.notifiers, stg_name) if config.live.notifiers else None
 
     # Create strategy initializer
     _initializer = BasicStrategyInitializer()
@@ -304,10 +342,6 @@ def create_strategy_context(
     _chan = CtrlChannel("databus", sentinel=(None, None, None, None))
     _sched = BasicScheduler(_chan, lambda: _time.time().item())
 
-    # Create time provider for metric emitters
-    if _metric_emitter is not None:
-        _metric_emitter.set_time_provider(_time)
-
     # Create health metrics monitor with emitter
     _health_monitor = BaseHealthMonitor(
         _time, emitter=_metric_emitter, channel=_chan, **config.live.health.model_dump()
@@ -331,6 +365,7 @@ def create_strategy_context(
                 channel=_chan,
                 account_manager=account_manager,
                 health_monitor=_health_monitor,
+                loop=loop,
             )
         )
         _exchange_to_account[exchange_name] = (
@@ -342,8 +377,10 @@ def create_strategy_context(
                 account_manager=account_manager,
                 tcc=tcc,
                 paper=paper,
+                data_provider=data_provider,
                 restored_state=restored_state,
                 read_only=config.live.read_only,
+                loop=loop,
             )
         )
         _exchange_to_broker[exchange_name] = _create_broker(
@@ -355,9 +392,26 @@ def create_strategy_context(
             data_provider=data_provider,
             account_manager=account_manager,
             paper=paper,
+            loop=loop,
         )
         _instruments.extend(_create_instruments_for_exchange(exchange_name, exchange_config))
 
+    # Use provided aux_configs or resolve if not provided (for backwards compatibility)
+    if aux_configs is None:
+        aux_configs = resolve_aux_config(config.aux, getattr(config.live, "aux", None))
+
+    _extend_aux_configs(aux_configs, list(_exchange_to_data_provider.values()))
+    _aux_reader = construct_aux_reader(aux_configs)
+
+    if _aux_reader is not None and config.live.prefetch:
+        prefetch_config = config.live.prefetch
+        if prefetch_config.enabled:
+            _aux_reader = CachedPrefetchReader(
+                _aux_reader,
+                prefetch_period=prefetch_config.prefetch_period,
+                cache_size_mb=prefetch_config.cache_size_mb,
+            )
+
     _account = (
         CompositeAccountProcessor(_time, _exchange_to_account)
         if len(exchanges) > 1
@@ -368,6 +422,9 @@ def create_strategy_context(
     # Create exporters if configured
     _exporter = create_exporters(config.live.exporters, stg_name, _account) if config.live.exporters else None
 
+    # Create data throttler from config
+    _data_throttler = _create_data_throttler(config.live.throttling) if config.live.throttling else None
+
     logger.info(f"- Strategy: <blue>{stg_name}</blue>\n- Mode: {_run_mode}\n- Parameters: {config.parameters}")
 
     ctx = StrategyContext(
@@ -383,13 +440,22 @@ def create_strategy_context(
         aux_data_provider=_aux_reader,
         exporter=_exporter,
         emitter=_metric_emitter,
-        lifecycle_notifier=_lifecycle_notifier,
+        notifier=_notifier,
         initializer=_initializer,
         strategy_name=stg_name,
         health_monitor=_health_monitor,
         restored_state=restored_state,
+        data_throttler=_data_throttler,
     )
 
+    # Store the shared event loop reference for cleanup
+    if loop is not None:
+        ctx._shared_event_loop = loop  # type: ignore
+
+    # Set context for metric emitters to enable is_live tag and time access
+    if _metric_emitter is not None:
+        _metric_emitter.set_context(ctx)
+
     return ctx
 
 
@@ -404,6 +470,31 @@ def _get_strategy_name(cfg: StrategyConfig) -> str:
         return cfg.strategy.__class__.__name__
 
 
+def _extend_aux_configs(aux_configs: list[ReaderConfig], data_providers: list[IDataProvider]):
+    reader_to_kwargs = {}
+    for data_provider in data_providers:
+        # Check if this is an xlighter data provider and extract the client
+        from qubx.connectors.xlighter.data import LighterDataProvider
+
+        if isinstance(data_provider, LighterDataProvider):
+            reader_to_kwargs["xlighter"] = {"client": data_provider.client}
+
+    for aux_config in aux_configs:
+        if aux_config.reader in reader_to_kwargs:
+            aux_config.args.update(reader_to_kwargs[aux_config.reader])
+
+
+def _extend_warmup_configs(typed_reader_configs: list[TypedReaderConfig], data_providers: list[IDataProvider]):
+    """
+    Inject clients from data providers into warmup reader configs.
+
+    Warmup uses TypedReaderConfig which contains nested ReaderConfig objects,
+    so we iterate through and inject clients into each nested reader list.
+    """
+    for typed_config in typed_reader_configs:
+        _extend_aux_configs(typed_config.readers, data_providers)
+
+
 def _setup_strategy_logging(
     stg_name: str,
     log_config: LoggingConfig,
@@ -462,11 +553,14 @@ def _create_data_provider(
     channel: CtrlChannel,
     account_manager: AccountConfigurationManager,
     health_monitor: IHealthMonitor | None = None,
+    loop: asyncio.AbstractEventLoop | None = None,
 ) -> IDataProvider:
     settings = account_manager.get_exchange_settings(exchange_name)
     match exchange_config.connector.lower():
         case "ccxt":
-            exchange_manager = get_ccxt_exchange_manager(exchange_name, use_testnet=settings.testnet)
+            exchange_manager = get_ccxt_exchange_manager(
+                exchange_name, use_testnet=settings.testnet, loop=loop, **exchange_config.params
+            )
             return CcxtDataProvider(
                 exchange_manager=exchange_manager,
                 time_provider=time_provider,
@@ -481,6 +575,28 @@ def _create_data_provider(
                 time_provider=time_provider,
                 channel=channel,
                 health_monitor=health_monitor,
+                asyncio_loop=loop,
+            )
+        case "xlighter":
+            from qubx.connectors.xlighter.websocket import LighterWebSocketManager
+
+            creds = account_manager.get_exchange_credentials(exchange_name)
+            client = get_xlighter_client(
+                api_key=creds.api_key,
+                secret=creds.secret,
+                account_index=cast(int, creds.get_extra_field("account_index")),
+                api_key_index=creds.get_extra_field("api_key_index"),
+                testnet=settings.testnet,
+                loop=loop,
+            )
+            # Create shared WebSocket manager and pass it to data provider
+            # Account and broker will reuse the same ws_manager from data_provider
+            ws_manager = LighterWebSocketManager(client=client, testnet=settings.testnet)
+            return get_xlighter_data_provider(
+                client=client,
+                time_provider=time_provider,
+                channel=channel,
+                ws_manager=ws_manager,
             )
         case _:
             raise ValueError(f"Connector {exchange_config.connector} is not supported yet !")
@@ -494,8 +610,10 @@ def _create_account_processor(
     account_manager: AccountConfigurationManager,
     tcc: TransactionCostsCalculator,
     paper: bool,
+    data_provider: IDataProvider | None = None,
     restored_state: RestoredState | None = None,
     read_only: bool = False,
+    loop: asyncio.AbstractEventLoop | None = None,
 ) -> IAccountProcessor:
     if paper:
         connector = "paper"
@@ -508,7 +626,7 @@ def _create_account_processor(
         case "ccxt":
             creds = account_manager.get_exchange_credentials(exchange_name)
             exchange_manager = get_ccxt_exchange_manager(
-                exchange_name, use_testnet=creds.testnet, api_key=creds.api_key, secret=creds.secret
+                exchange_name, use_testnet=creds.testnet, api_key=creds.api_key, secret=creds.secret, loop=loop
             )
             return get_ccxt_account(
                 exchange_name,
@@ -520,6 +638,26 @@ def _create_account_processor(
                 tcc=tcc,
                 read_only=read_only,
             )
+        case "xlighter":
+            from qubx.connectors.xlighter.data import LighterDataProvider
+
+            creds = account_manager.get_exchange_credentials(exchange_name)
+
+            # Get client and ws_manager from data_provider to ensure resource sharing
+            assert isinstance(data_provider, LighterDataProvider), "Data provider must be LighterDataProvider"
+            client = data_provider.client
+            ws_manager = data_provider.ws_manager
+            instrument_loader = data_provider.instrument_loader
+
+            return get_xlighter_account(
+                client=client,
+                channel=channel,
+                time_provider=time_provider,
+                base_currency=creds.base_currency,
+                initial_capital=creds.initial_capital,
+                ws_manager=ws_manager,
+                instrument_loader=instrument_loader,
+            )
         case "paper":
             settings = account_manager.get_exchange_settings(exchange_name)
 
@@ -547,6 +685,7 @@ def _create_broker(
     data_provider: IDataProvider,
     account_manager: AccountConfigurationManager,
     paper: bool,
+    loop: asyncio.AbstractEventLoop | None = None,
 ) -> IBroker:
     if paper:
         connector = "paper"
@@ -568,10 +707,26 @@ def _create_broker(
                 api_key=creds.api_key,
                 secret=creds.secret,
                 enable_mm=_enable_mm,
+                loop=loop,
             )
             return get_ccxt_broker(
                 exchange_name, exchange_manager, channel, time_provider, account, data_provider, **params
             )
+        case "xlighter":
+            # For xlighter, get client, ws_manager, and instrument_loader from data_provider
+            from qubx.connectors.xlighter.data import LighterDataProvider
+
+            assert isinstance(data_provider, LighterDataProvider), "Data provider must be LighterDataProvider"
+            return get_xlighter_broker(
+                client=data_provider.client,
+                channel=channel,
+                time_provider=time_provider,
+                account=account,
+                data_provider=data_provider,
+                ws_manager=data_provider.ws_manager,
+                instrument_loader=data_provider.instrument_loader,
+                **params,
+            )
         case "paper":
             assert isinstance(account, SimulatedAccountProcessor)
             return SimulatedBroker(channel=channel, account=account, simulated_exchange=account._exchange)
@@ -590,6 +745,36 @@ def _create_instruments_for_exchange(exchange_name: str, exchange_config: Exchan
     return instruments
 
 
+def _create_data_throttler(throttling_config):
+    """
+    Create data throttler from throttling configuration.
+
+    Args:
+        throttling_config: ThrottlingConfig object with throttle settings
+
+    Returns:
+        InstrumentThrottler configured with per-data-type frequency limits, or None if disabled
+    """
+    from qubx.utils.throttler import InstrumentThrottler
+
+    if not throttling_config or not throttling_config.enabled:
+        return None
+
+    # Build config dict: {data_type: max_frequency_hz}
+    throttle_cfg_dict = {}
+    for throttle_cfg in throttling_config.throttles:
+        if throttle_cfg.enabled:
+            throttle_cfg_dict[throttle_cfg.data_type] = throttle_cfg.max_frequency_hz
+            logger.info(
+                f"Throttling <y>{throttle_cfg.data_type}</y>: max {throttle_cfg.max_frequency_hz} Hz per instrument"
+            )
+
+    if not throttle_cfg_dict:
+        return None
+
+    return InstrumentThrottler(throttle_cfg_dict)
+
+
 def _apply_inverse_exchange_mapping(exchanges: list[str]) -> list[str]:
     """
     Apply inverse exchange mapping to the list of exchanges.
@@ -611,7 +796,7 @@ def _run_warmup(
     restored_state: RestoredState | None,
     exchanges: dict[str, ExchangeConfig],
     warmup: WarmupConfig | None,
-    aux_configs: list[ReaderConfig],
+    prefetch_config: PrefetchConfig,
     simulated_formatter: SimulatedLogFormatter,
     enable_funding: bool = False,
 ) -> None:
@@ -650,15 +835,16 @@ def _run_warmup(
     logger.info(f"<yellow>Warmup start time: {warmup_start_time}</yellow>")
 
     # - construct warmup readers
+    # Inject clients into warmup reader configs before creating readers
+    if warmup and warmup.readers and hasattr(ctx, "_data_providers"):
+        _extend_warmup_configs(warmup.readers, list(ctx._data_providers))  # type: ignore
+
     data_type_to_reader = create_data_type_readers(warmup.readers) if warmup else {}
 
     if not data_type_to_reader:
         logger.warning("<yellow>No readers were created for warmup</yellow>")
         return
 
-    # Use the provided aux_configs (already resolved with live section override)
-    _aux_reader = construct_aux_reader(aux_configs)
-
     # - create instruments
     instruments = []
     for exchange_name, exchange_config in exchanges.items():
@@ -673,13 +859,13 @@ def _run_warmup(
     warmup_runner = SimulationRunner(
         setup=SimulationSetup(
             setup_type=SetupTypes.STRATEGY,
-            name=getattr(ctx, "_strategy_name", ctx.strategy.__class__.__name__),
+            name=ctx.strategy_name,
             generator=ctx.strategy,
             tracker=None,
             instruments=instruments,
             # Apply inverse exchange mapping so SimulationRunner doesn't need EXCHANGE_MAPPINGS
             exchanges=_apply_inverse_exchange_mapping(ctx.exchanges),
-            capital=ctx.account.get_capital(),
+            capital=ctx.account.get_total_capital(),
             base_currency=ctx.account.get_base_currency(),
             commissions=None,  # TODO: get commissions from somewhere
             enable_funding=enable_funding,
@@ -687,12 +873,13 @@ def _run_warmup(
         data_config=recognize_simulation_data_config(
             decls=data_type_to_reader,  # type: ignore
             instruments=instruments,
-            aux_data=_aux_reader,
-            prefetch_config=warmup.prefetch,
+            aux_data=ctx.aux,
+            prefetch_config=prefetch_config,
         ),
-        start=pd.Timestamp(warmup_start_time),
-        stop=pd.Timestamp(current_time),
+        start=cast(pd.Timestamp, pd.Timestamp(warmup_start_time)),
+        stop=cast(pd.Timestamp, pd.Timestamp(current_time)),
         emitter=ctx.emitter,
+        notifier=ctx.notifier,
         strategy_state=ctx._strategy_state,
         initializer=ctx.initializer,
         warmup_mode=True,
@@ -709,9 +896,9 @@ def _run_warmup(
     finally:
         # Restore the live time provider
         simulated_formatter.time_provider = _live_time_provider
-        # Set back the time provider for metric emitters to use live time provider
+        # Set back the context for metric emitters to use live context
         if ctx.emitter is not None:
-            ctx.emitter.set_time_provider(_live_time_provider)
+            ctx.emitter.set_context(ctx)
         ctx._strategy_state.is_warmup_in_progress = False
         ctx.initializer.simulation = False
 
@@ -803,7 +990,7 @@ def simulate_strategy(
 
     stg = cfg.strategy
     simulation_name = config_file.stem
-    _v_id = pd.Timestamp("now").strftime("%Y%m%d%H%M%S")
+    _v_id = cast(pd.Timestamp, pd.Timestamp("now")).strftime("%Y%m%d%H%M%S")
 
     match stg:
         case list():
@@ -850,9 +1037,12 @@ def simulate_strategy(
         "enable_inmemory_emitter": cfg.simulation.enable_inmemory_emitter,
     }
 
+    if cfg.simulation.base_currency is not None:
+        sim_params["base_currency"] = cfg.simulation.base_currency
+
     if cfg.simulation.debug is not None:
         sim_params["debug"] = cfg.simulation.debug
-    
+
     if cfg.simulation.portfolio_log_freq is not None:
         sim_params["portfolio_log_freq"] = cfg.simulation.portfolio_log_freq
 
